"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
/*
 * Copyright (c) 2020, salesforce.com, inc.
 * All rights reserved.
 * Licensed under the BSD 3-Clause license.
 * For full license text, see LICENSE.txt file in the repo root or https://opensource.org/licenses/BSD-3-Clause
 */
const types_1 = require("./types");
const convert_1 = require("../convert");
const errors_1 = require("../errors");
const metadata_registry_1 = require("../metadata-registry");
const diagnosticUtil_1 = require("./diagnosticUtil");
const unzipper = require("unzipper");
const fast_xml_parser_1 = require("fast-xml-parser");
const stream_1 = require("stream");
const util_1 = require("util");
const path_1 = require("path");
const os_1 = require("os");
const fs_1 = require("fs");
const fileSystemHandler_1 = require("../utils/fileSystemHandler");
const pipeline = util_1.promisify(stream_1.pipeline);
exports.DEFAULT_API_OPTIONS = {
    rollbackOnError: true,
    ignoreWarnings: false,
    checkOnly: false,
    singlePackage: true,
};
class MetadataApi extends types_1.BaseApi {
    // TODO: move filtering logic to registry W-8023153
    retrieveWithPaths(options) {
        return __awaiter(this, void 0, void 0, function* () {
            const allComponents = [];
            for (const filepath of options.paths) {
                allComponents.push(...this.registry.getComponentsFromPath(filepath));
            }
            const hashedCmps = new Set();
            const uniqueComponents = allComponents.filter((component) => {
                const hashed = this.hashElement(component);
                if (!hashedCmps.has(hashed)) {
                    hashedCmps.add(hashed);
                    return component;
                }
            });
            const retrieveOptions = { components: uniqueComponents };
            return this.retrieve(Object.assign(retrieveOptions, options));
        });
    }
    retrieve(options) {
        return __awaiter(this, void 0, void 0, function* () {
            let components = [];
            const retrieveRequest = this.formatRetrieveRequest(options.components);
            const retrieveResult = yield this.getRetrievedResult(retrieveRequest, options);
            if (retrieveResult.status === types_1.RetrieveStatus.Succeeded) {
                const extractedComponents = yield this.extractComponents(retrieveResult);
                components = yield this.getConvertedComponents(extractedComponents, options);
            }
            const sourceRetrieveResult = this.buildSourceRetrieveResult(retrieveResult, components);
            return sourceRetrieveResult;
        });
    }
    formatRetrieveRequest(components) {
        const manifestGenerator = new metadata_registry_1.ManifestGenerator(this.registry);
        const manifest = manifestGenerator.createManifest(components);
        const manifestJson = fast_xml_parser_1.parse(manifest);
        const packageData = manifestJson.Package;
        delete packageData.$;
        const retrieveRequest = {
            apiVersion: this.registry.getApiVersion(),
            unpackaged: packageData,
        };
        return retrieveRequest;
    }
    getRetrievedResult(retrieveRequest, options) {
        return __awaiter(this, void 0, void 0, function* () {
            // @ts-ignore required callback
            const retrieveId = (yield this.connection.metadata.retrieve(retrieveRequest)).id;
            const retrieveResult = yield this.metadataRetrieveStatusPoll(retrieveId, options);
            return retrieveResult;
        });
    }
    metadataRetrieveStatusPoll(retrieveID, options, interval = 100) {
        return __awaiter(this, void 0, void 0, function* () {
            let result;
            const wait = (interval) => {
                return new Promise((resolve) => {
                    setTimeout(resolve, interval);
                });
            };
            const timeout = !options || !options.wait ? 10000 : options.wait;
            const endTime = Date.now() + timeout;
            let triedOnce = false;
            do {
                if (triedOnce) {
                    yield wait(interval);
                }
                try {
                    // Recasting to use the library's RetrieveResult type
                    result = (yield this.connection.metadata.checkRetrieveStatus(retrieveID));
                }
                catch (e) {
                    throw new errors_1.RetrieveError('md_request_fail', e);
                }
                switch (result.status) {
                    case types_1.RetrieveStatus.Succeeded:
                    case types_1.RetrieveStatus.Failed:
                        return result;
                    case types_1.RetrieveStatus.InProgress:
                }
                triedOnce = true;
            } while (Date.now() < endTime);
            return result;
        });
    }
    buildSourceRetrieveResult(retrieveResult, components) {
        const sourceRetrieveResult = {
            status: retrieveResult.status,
            id: retrieveResult.id,
            message: retrieveResult.messages,
            success: retrieveResult.status === types_1.RetrieveStatus.Succeeded ? true : false,
            components,
        };
        return sourceRetrieveResult;
    }
    extractComponents(retrieveResult) {
        return __awaiter(this, void 0, void 0, function* () {
            const tempCmpDir = path_1.join(os_1.tmpdir(), '.sfdx', 'tmp');
            fileSystemHandler_1.deleteDirectory(tempCmpDir);
            fileSystemHandler_1.ensureDirectoryExists(tempCmpDir);
            const tmpZip = path_1.join(tempCmpDir, 'metadataformat.zip');
            fs_1.writeFileSync(tmpZip, retrieveResult.zipFile, 'base64');
            const retrieveStream = fs_1.createReadStream(tmpZip);
            const outputStream = unzipper.Extract({ path: tempCmpDir });
            return new Promise((resolve, reject) => {
                pipeline(retrieveStream, outputStream).catch((err) => reject(err));
                outputStream.on('close', () => {
                    const retrievedComponents = this.registry.getComponentsFromPath(tempCmpDir);
                    resolve(retrievedComponents);
                });
            });
        });
    }
    getConvertedComponents(retrievedComponents, options) {
        return __awaiter(this, void 0, void 0, function* () {
            const converter = new convert_1.MetadataConverter();
            const convertResult = yield converter.convert(retrievedComponents, 'source', {
                type: 'directory',
                outputDirectory: options.output,
            });
            const convertedComponents = this.registry.getComponentsFromPath(convertResult.packagePath);
            return convertedComponents;
        });
    }
    hashElement(component) {
        const hashed = `${component.fullName}.${component.type.id}`;
        return hashed;
    }
    deploy(components, options) {
        return __awaiter(this, void 0, void 0, function* () {
            const metadataComponents = Array.isArray(components) ? components : [components];
            const converter = new convert_1.MetadataConverter();
            const { zipBuffer } = yield converter.convert(metadataComponents, 'metadata', { type: 'zip' });
            const deployID = yield this.metadataDeployID(zipBuffer, options);
            const deployStatusPoll = this.metadataDeployStatusPoll(deployID, options);
            const componentDeploymentMap = new Map();
            for (const component of metadataComponents) {
                componentDeploymentMap.set(`${component.type.name}:${component.fullName}`, {
                    status: types_1.ComponentStatus.Unchanged,
                    component,
                    diagnostics: [],
                });
            }
            const result = yield deployStatusPoll;
            return this.buildSourceDeployResult(componentDeploymentMap, result);
        });
    }
    deployWithPaths(paths, options) {
        return __awaiter(this, void 0, void 0, function* () {
            const components = this.registry.getComponentsFromPath(paths);
            return this.deploy(components, options);
        });
    }
    metadataDeployID(zipBuffer, options) {
        return __awaiter(this, void 0, void 0, function* () {
            if (!options || !options.apiOptions) {
                options = {
                    apiOptions: exports.DEFAULT_API_OPTIONS,
                };
            }
            else {
                for (const [property, value] of Object.entries(exports.DEFAULT_API_OPTIONS)) {
                    if (!(property in options.apiOptions)) {
                        //@ts-ignore ignore while dynamically building the defaults
                        options.apiOptions[property] = value;
                    }
                }
            }
            const result = yield this.connection.metadata.deploy(zipBuffer, options.apiOptions);
            return result.id;
        });
    }
    metadataDeployStatusPoll(deployID, options, interval = 100) {
        return __awaiter(this, void 0, void 0, function* () {
            let result;
            const wait = (interval) => {
                return new Promise((resolve) => {
                    setTimeout(resolve, interval);
                });
            };
            const timeout = !options || !options.wait ? 10000 : options.wait;
            const endTime = Date.now() + timeout;
            let triedOnce = false;
            do {
                if (triedOnce) {
                    yield wait(interval);
                }
                try {
                    // Recasting to use the library's DeployResult type
                    result = (yield this.connection.metadata.checkDeployStatus(deployID, true));
                }
                catch (e) {
                    throw new errors_1.DeployError('md_request_fail', e);
                }
                switch (result.status) {
                    case types_1.DeployStatus.Succeeded:
                    case types_1.DeployStatus.Failed:
                    case types_1.DeployStatus.Canceled:
                        return result;
                }
                triedOnce = true;
            } while (Date.now() < endTime);
            return result;
        });
    }
    buildSourceDeployResult(componentDeploymentMap, result) {
        const deployResult = {
            id: result.id,
            status: result.status,
            success: result.success,
        };
        const messages = this.getDeployMessages(result);
        const diagnosticUtil = new diagnosticUtil_1.DiagnosticUtil('metadata');
        if (messages.length > 0) {
            deployResult.components = [];
            for (let message of messages) {
                message = this.sanitizeDeployMessage(message);
                const componentKey = `${message.componentType}:${message.fullName}`;
                const componentDeployment = componentDeploymentMap.get(componentKey);
                if (componentDeployment) {
                    if (message.created === 'true') {
                        componentDeployment.status = types_1.ComponentStatus.Created;
                    }
                    else if (message.changed === 'true') {
                        componentDeployment.status = types_1.ComponentStatus.Changed;
                    }
                    else if (message.deleted === 'true') {
                        componentDeployment.status = types_1.ComponentStatus.Deleted;
                    }
                    else if (message.success === 'false') {
                        componentDeployment.status = types_1.ComponentStatus.Failed;
                    }
                    if (message.problem) {
                        diagnosticUtil.setDiagnostic(componentDeployment, message);
                    }
                }
            }
            deployResult.components = Array.from(componentDeploymentMap.values());
        }
        return deployResult;
    }
    getDeployMessages(result) {
        const messages = [];
        if (result.details) {
            const { componentSuccesses, componentFailures } = result.details;
            if (componentSuccesses) {
                if (Array.isArray(componentSuccesses)) {
                    messages.push(...componentSuccesses);
                }
                else {
                    messages.push(componentSuccesses);
                }
            }
            if (componentFailures) {
                if (Array.isArray(componentFailures)) {
                    messages.push(...componentFailures);
                }
                else {
                    messages.push(componentFailures);
                }
            }
        }
        return messages;
    }
    /**
     * Fix any issues with the deploy message returned by the api.
     * TODO: remove as fixes are made in the api.
     */
    sanitizeDeployMessage(message) {
        // lwc doesn't properly use the fullname property in the api.
        message.fullName = message.fullName.replace(/markup:\/\/c:/, '');
        return message;
    }
}
exports.MetadataApi = MetadataApi;
//# sourceMappingURL=metadataApi.js.map