"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
/*
 * Copyright (c) 2020, salesforce.com, inc.
 * All rights reserved.
 * Licensed under the BSD 3-Clause license.
 * For full license text, see LICENSE.txt file in the repo root or https://opensource.org/licenses/BSD-3-Clause
 */
const baseMetadataTransformer_1 = require("./baseMetadataTransformer");
const archiver_1 = require("archiver");
const mime_1 = require("mime");
const unzipper_1 = require("unzipper");
const path_1 = require("path");
const utils_1 = require("../../utils");
const fs_1 = require("fs");
const errors_1 = require("../../errors");
const constants_1 = require("../../utils/constants");
class StaticResourceMetadataTransformer extends baseMetadataTransformer_1.BaseMetadataTransformer {
    toMetadataFormat(component) {
        let contentSource;
        const { content, type, xml } = component;
        const writerFormat = {
            component,
            writeInfos: [],
        };
        if (this.componentIsExpandedArchive(component)) {
            const zip = archiver_1.create('zip', { zlib: { level: 3 } });
            zip.directory(content, false);
            zip.finalize();
            contentSource = zip;
        }
        else {
            contentSource = fs_1.createReadStream(content);
        }
        writerFormat.writeInfos.push({
            source: contentSource,
            relativeDestination: path_1.join(type.directoryName, `${utils_1.baseName(content)}.${type.suffix}`),
        }, {
            source: fs_1.createReadStream(xml),
            relativeDestination: path_1.join(type.directoryName, path_1.basename(xml)),
        });
        return writerFormat;
    }
    toSourceFormat(component) {
        const { xml, type, content } = component;
        const result = { component, writeInfos: [] };
        if (content) {
            const contentType = this.getContentType(component);
            if (constants_1.ARCHIVE_MIME_TYPES.has(contentType)) {
                const baseDir = path_1.join(type.directoryName, utils_1.baseName(content));
                this.createWriteInfosFromArchive(content, baseDir, result);
            }
            else {
                const extension = this.getExtensionFromType(contentType);
                result.writeInfos.push({
                    source: fs_1.createReadStream(content),
                    relativeDestination: path_1.join(type.directoryName, `${utils_1.baseName(content)}.${extension}`),
                });
            }
            result.writeInfos.push({
                source: fs_1.createReadStream(xml),
                relativeDestination: path_1.join(type.directoryName, path_1.basename(xml)),
            });
        }
        return result;
    }
    componentIsExpandedArchive(component) {
        const { content, tree } = component;
        if (tree.isDirectory(content)) {
            const contentType = this.getContentType(component);
            if (constants_1.ARCHIVE_MIME_TYPES.has(contentType)) {
                return true;
            }
            throw new errors_1.LibraryError('error_static_resource_expected_archive_type', [
                contentType,
                component.name,
            ]);
        }
        return false;
    }
    getContentType(component) {
        return component.parseXml().StaticResource.contentType;
    }
    getExtensionFromType(contentType) {
        let ext = mime_1.getExtension(contentType);
        if (!ext && constants_1.FALLBACK_TYPE_MAP.get(contentType)) {
            ext = constants_1.FALLBACK_TYPE_MAP.get(contentType);
        }
        // return registered ext, fallback, or the default (application/octet-stream -> bin)
        return ext || mime_1.getExtension(constants_1.DEFAULT_CONTENT_TYPE);
    }
    createWriteInfosFromArchive(zipPath, destDir, format) {
        format.getExtraInfos = () => __awaiter(this, void 0, void 0, function* () {
            const writeInfos = [];
            const directory = yield unzipper_1.Open.file(zipPath);
            directory.files.forEach((f) => {
                if (f.type === 'File') {
                    writeInfos.push({
                        source: f.stream(),
                        relativeDestination: path_1.join(destDir, f.path),
                    });
                }
            });
            return writeInfos;
        });
    }
}
exports.StaticResourceMetadataTransformer = StaticResourceMetadataTransformer;
//# sourceMappingURL=staticResourceMetadataTransformer.js.map