"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const path_1 = require("path");
const utils_1 = require("../utils");
const registry_1 = require("../utils/registry");
const fs_1 = require("fs");
const errors_1 = require("../errors");
/**
 * An extendable base class for implementing the `TreeContainer` interface
 */
class BaseTreeContainer {
    find(fileType, fullName, dir) {
        const fileName = this.readDirectory(dir).find((entry) => {
            const parsed = registry_1.parseMetadataXml(path_1.join(dir, entry));
            const metaXmlCondition = fileType === 'metadata' ? !!parsed : !parsed;
            return utils_1.baseName(entry) === fullName && metaXmlCondition;
        });
        if (fileName) {
            return path_1.join(dir, fileName);
        }
    }
}
exports.BaseTreeContainer = BaseTreeContainer;
class NodeFSTreeContainer extends BaseTreeContainer {
    isDirectory(fsPath) {
        return fs_1.lstatSync(fsPath).isDirectory();
    }
    exists(fsPath) {
        return fs_1.existsSync(fsPath);
    }
    readDirectory(fsPath) {
        return fs_1.readdirSync(fsPath);
    }
    readFile(fsPath) {
        return fs_1.promises.readFile(fsPath);
    }
}
exports.NodeFSTreeContainer = NodeFSTreeContainer;
class VirtualTreeContainer extends BaseTreeContainer {
    constructor(virtualFs) {
        super();
        this.tree = new Map();
        this.fileContents = new Map();
        this.populate(virtualFs);
    }
    isDirectory(fsPath) {
        if (this.exists(fsPath)) {
            return this.tree.has(fsPath);
        }
        throw new errors_1.LibraryError('error_path_not_found', fsPath);
    }
    exists(fsPath) {
        const files = this.tree.get(path_1.dirname(fsPath));
        const isFile = files && files.has(fsPath);
        return isFile || this.tree.has(fsPath);
    }
    readDirectory(fsPath) {
        return Array.from(this.tree.get(fsPath)).map((p) => path_1.basename(p));
    }
    readFile(fsPath) {
        if (this.exists(fsPath)) {
            let data = this.fileContents.get(fsPath);
            if (!data) {
                data = Buffer.from('');
                this.fileContents.set(fsPath, data);
            }
            return Promise.resolve(data);
        }
        throw new errors_1.LibraryError('error_path_not_found', fsPath);
    }
    populate(virtualFs) {
        for (const dir of virtualFs) {
            const { dirPath, children } = dir;
            this.tree.set(dirPath, new Set());
            for (const child of children) {
                let childPath;
                let childData;
                if (typeof child === 'string') {
                    childPath = path_1.join(dirPath, child);
                }
                else {
                    childPath = path_1.join(dirPath, child.name);
                    childData = child.data;
                }
                this.tree.get(dirPath).add(childPath);
                if (childData) {
                    this.fileContents.set(childPath, childData);
                }
            }
        }
    }
}
exports.VirtualTreeContainer = VirtualTreeContainer;
//# sourceMappingURL=treeContainers.js.map