"use strict";
/*
 * Copyright (c) 2020, salesforce.com, inc.
 * All rights reserved.
 * Licensed under the BSD 3-Clause license.
 * For full license text, see LICENSE.txt file in the repo root or https://opensource.org/licenses/BSD-3-Clause
 */
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const core_1 = require("@salesforce/core");
const testSetup_1 = require("@salesforce/core/lib/testSetup");
const chai_1 = require("chai");
const fs = require("fs");
const path_1 = require("path");
const sinon_1 = require("sinon");
const i18n_1 = require("../../../src/i18n");
const client_1 = require("../../../src/client");
const auraDeployMocks_1 = require("./auraDeployMocks");
const deployStrategies_1 = require("../../../src/client/deployStrategies");
const $$ = testSetup_1.testSetup();
describe('Aura Deploy Strategy', () => {
    const testMetadataField = {
        apiVersion: '32.0',
        status: 'Active',
    };
    const testData = new testSetup_1.MockTestOrgData();
    let mockConnection;
    let sandboxStub;
    let simpleMetaXMLString = '<?xml version="1.0" encoding="UTF-8"?>';
    simpleMetaXMLString += '<ApexClass xmlns="http://soap.sforce.com/2006/04/metadata">';
    simpleMetaXMLString += '    <apiVersion>32.0</apiVersion>';
    simpleMetaXMLString += '    <status>Active</status>';
    simpleMetaXMLString += '</ApexClass>';
    beforeEach(() => __awaiter(void 0, void 0, void 0, function* () {
        sandboxStub = sinon_1.createSandbox();
        $$.setConfigStubContents('AuthInfoConfig', {
            contents: yield testData.getConfig(),
        });
        mockConnection = yield core_1.Connection.create({
            authInfo: yield core_1.AuthInfo.create({
                username: testData.username,
            }),
        });
        const mockFS = sandboxStub.stub(fs, 'readFileSync');
        mockFS.withArgs(auraDeployMocks_1.auraFiles[0], 'utf8').returns(auraDeployMocks_1.auraContents[0]);
        mockFS.withArgs(auraDeployMocks_1.auraFiles[1], 'utf8').returns(auraDeployMocks_1.auraContents[1]);
        mockFS.withArgs(auraDeployMocks_1.auraFiles[2], 'utf8').returns(auraDeployMocks_1.auraContents[2]);
        mockFS.withArgs(auraDeployMocks_1.auraFiles[3], 'utf8').returns(auraDeployMocks_1.auraContents[3]);
        mockFS.withArgs(auraDeployMocks_1.auraFiles[4], 'utf8').returns(auraDeployMocks_1.auraContents[4]);
        mockFS.withArgs(auraDeployMocks_1.auraFiles[5], 'utf8').returns(auraDeployMocks_1.auraContents[5]);
        mockFS.withArgs(auraDeployMocks_1.auraFiles[6], 'utf8').returns(auraDeployMocks_1.auraContents[6]);
        mockFS.withArgs(auraDeployMocks_1.auraFiles[7], 'utf8').returns(auraDeployMocks_1.auraContents[7]);
        mockFS
            .withArgs(path_1.join('file', 'path', 'aura', 'mockAuraCmp', 'mockAuraCmp.cmp-meta.xml'), 'utf8')
            .returns(simpleMetaXMLString);
    }));
    afterEach(() => {
        sandboxStub.restore();
    });
    it('should build list of aura definition objects with correct properties', () => __awaiter(void 0, void 0, void 0, function* () {
        sandboxStub
            .stub(mockConnection.tooling, 'query')
            // @ts-ignore
            .resolves({ records: [] });
        const mockToolingCreate = sandboxStub.stub(mockConnection.tooling, 'create');
        mockToolingCreate.resolves({
            success: true,
            id: '1dcxxx000000060',
            errors: [],
        });
        const auraDeploy = new deployStrategies_1.AuraDeploy(mockConnection);
        auraDeploy.component = auraDeployMocks_1.auraComponent;
        const auraDefinitions = yield auraDeploy.buildDefList();
        chai_1.expect(auraDefinitions).to.deep.include.members(auraDeployMocks_1.testAuraList);
    }));
    it('should filter existing sources by def type and attach id property for matches', () => __awaiter(void 0, void 0, void 0, function* () {
        const mockToolingQuery = sandboxStub.stub(mockConnection.tooling, 'query');
        const matches = [
            {
                Id: '1dcxxx000000034',
                DefType: 'COMPONENT',
                Format: 'XML',
                FilePath: auraDeployMocks_1.auraFiles[1],
                Source: auraDeployMocks_1.auraContents[1],
            },
            {
                Id: '1dcxxx000000035',
                DefType: 'STYLE',
                Format: 'CSS',
                FilePath: auraDeployMocks_1.auraFiles[2],
                Source: auraDeployMocks_1.auraContents[2],
            },
            {
                Id: '1dcxxx000000036',
                DefType: 'DESIGN',
                Format: 'XML',
                FilePath: auraDeployMocks_1.auraFiles[3],
                Source: auraDeployMocks_1.auraContents[3],
            },
        ];
        // @ts-ignore
        mockToolingQuery.resolves({ records: matches });
        sandboxStub.stub(deployStrategies_1.AuraDeploy.prototype, 'upsertBundle').resolves({
            success: true,
            id: '1dcxxx000000033',
            errors: [],
        });
        const auraDeploy = new deployStrategies_1.AuraDeploy(mockConnection);
        auraDeploy.component = auraDeployMocks_1.auraComponent;
        const auraResults = yield auraDeploy.buildDefList();
        chai_1.expect(auraResults).to.deep.include.members(matches);
    }));
    it('should filter existing sources and keep list unchanged if there are no matches', () => __awaiter(void 0, void 0, void 0, function* () {
        const mockToolingQuery = sandboxStub.stub(mockConnection.tooling, 'query');
        const matches = [
            {
                Id: '1dcxxx000000035',
                FilePath: path_1.join('path', 'to', 'wrong', 'aura', 'auraFile', 'auraFile.html'),
                Format: 'html',
                DefType: 'wrongType',
                Source: auraDeployMocks_1.auraContents[0],
                AuraDefinitionBundleId: '1dcxxx000000060',
            },
            {
                Id: '1dcxxx000000036',
                FilePath: 'path/to/wrong/aura/auraFile/auraFile.js',
                Format: 'js',
                DefType: 'wrongType',
                Source: auraDeployMocks_1.auraContents[1],
                AuraDefinitionBundleId: '1dcxxx000000060',
            },
        ];
        // @ts-ignore
        mockToolingQuery.resolves({ records: matches });
        sandboxStub.stub(deployStrategies_1.AuraDeploy.prototype, 'upsertBundle').resolves({
            success: true,
            id: '1dcxxx000000033',
            errors: [],
        });
        const auraDeploy = new deployStrategies_1.AuraDeploy(mockConnection);
        auraDeploy.component = auraDeployMocks_1.auraComponent;
        const auraResults = yield auraDeploy.buildDefList();
        chai_1.expect(auraResults).to.not.deep.include.members(matches);
    }));
    it('should create an auradefinitionbundle given the fullname and metadata', () => __awaiter(void 0, void 0, void 0, function* () {
        const mockToolingCreate = sandboxStub.stub(mockConnection.tooling, 'create');
        mockToolingCreate.resolves({
            success: true,
            id: '1dcxxx000000034',
            errors: [],
        });
        sandboxStub.stub(deployStrategies_1.AuraDeploy.prototype, 'buildMetadataField').returns(testMetadataField);
        const auraDeploy = new deployStrategies_1.AuraDeploy(mockConnection);
        auraDeploy.component = auraDeployMocks_1.auraComponent;
        const bundle = yield auraDeploy.upsertBundle();
        chai_1.expect(bundle.id).to.equal('1dcxxx000000034');
        chai_1.expect(bundle.success).to.be.equal(true);
        // tslint:disable-next-line:no-unused-expression
        chai_1.expect(bundle.errors).to.be.an('array').that.is.empty;
        chai_1.expect(mockToolingCreate.getCall(0).args[0]).to.equal(auraDeployMocks_1.auraComponent.type.name);
        chai_1.expect(mockToolingCreate.getCall(0).args[1]).to.be.an('object');
    }));
    it('should create an auradefinitionbundle given the fullname and metadata', () => __awaiter(void 0, void 0, void 0, function* () {
        const mockToolingUpdate = sandboxStub.stub(mockConnection.tooling, 'update');
        mockToolingUpdate.resolves({
            success: true,
            id: '1dcxxx000000034',
            errors: [],
        });
        sandboxStub.stub(deployStrategies_1.AuraDeploy.prototype, 'buildMetadataField').returns(testMetadataField);
        const auraDeploy = new deployStrategies_1.AuraDeploy(mockConnection);
        auraDeploy.component = auraDeployMocks_1.auraComponent;
        const bundle = yield auraDeploy.upsertBundle('1dcxxx000000034');
        chai_1.expect(bundle.id).to.equal('1dcxxx000000034');
        chai_1.expect(bundle.success).to.be.equal(true);
        // tslint:disable-next-line:no-unused-expression
        chai_1.expect(bundle.errors).to.be.an('array').that.is.empty;
        chai_1.expect(mockToolingUpdate.getCall(0).args[0]).to.equal(auraDeployMocks_1.auraComponent.type.name);
        chai_1.expect(mockToolingUpdate.getCall(0).args[1]).to.be.an('object');
    }));
    it('should throw an error if there is a problem creating the bundle', () => __awaiter(void 0, void 0, void 0, function* () {
        sandboxStub.stub(mockConnection.tooling, 'create').resolves({
            success: false,
            id: '',
            errors: ['Unexpected error while creating record'],
        });
        sandboxStub.stub(deployStrategies_1.AuraDeploy.prototype, 'buildMetadataField').returns(testMetadataField);
        const auraDeploy = new deployStrategies_1.AuraDeploy(mockConnection);
        auraDeploy.component = auraDeployMocks_1.auraComponent;
        try {
            yield auraDeploy.upsertBundle();
            chai_1.expect.fail('Should have failed');
        }
        catch (e) {
            chai_1.expect(e.message).to.equal(i18n_1.nls.localize('error_creating_metadata_type', 'AuraDefinitionBundle'));
            chai_1.expect(e.name).to.be.equal('DeployError');
        }
    }));
    it('should create sources in bundle and return successes in correct shape', () => __awaiter(void 0, void 0, void 0, function* () {
        const mockToolingCreate = sandboxStub.stub(mockConnection.tooling, 'create');
        mockToolingCreate.resolves({
            success: true,
            id: '1dcxxx000000034',
            errors: [],
        });
        sandboxStub.stub(deployStrategies_1.AuraDeploy.prototype, 'buildMetadataField').returns(testMetadataField);
        sandboxStub.stub(deployStrategies_1.AuraDeploy.prototype, 'buildDefList').resolves(auraDeployMocks_1.testAuraList);
        const mockToolingQuery = sandboxStub.stub(mockConnection.tooling, 'query');
        // @ts-ignore
        mockToolingQuery.resolves({ records: [] });
        const auraDeploy = new deployStrategies_1.AuraDeploy(mockConnection);
        const deployResults = yield auraDeploy.deploy(auraDeployMocks_1.auraComponent, '');
        chai_1.expect(deployResults).to.deep.equal({
            id: undefined,
            status: "Completed" /* Completed */,
            success: true,
            components: [
                {
                    status: client_1.ComponentStatus.Created,
                    component: auraDeployMocks_1.auraComponent,
                    diagnostics: [],
                },
            ],
        });
    }));
    it('should set component status to "created" when all definition files created', () => __awaiter(void 0, void 0, void 0, function* () {
        sandboxStub.stub(deployStrategies_1.AuraDeploy.prototype, 'buildDefList').resolves(auraDeployMocks_1.testAuraList);
        sandboxStub
            .stub(mockConnection.tooling, 'query')
            // @ts-ignore
            .resolves({ records: [] });
        sandboxStub.stub(mockConnection.tooling, 'create').resolves();
        const auraDeploy = new deployStrategies_1.AuraDeploy(mockConnection);
        const result = yield auraDeploy.deploy(auraDeployMocks_1.auraComponent, '');
        chai_1.expect(result).to.deep.equal({
            id: undefined,
            status: "Completed" /* Completed */,
            success: true,
            components: [
                {
                    status: client_1.ComponentStatus.Created,
                    component: auraDeployMocks_1.auraComponent,
                    diagnostics: [],
                },
            ],
        });
    }));
    it('should set component status to "changed" when one or more definition files updated', () => __awaiter(void 0, void 0, void 0, function* () {
        const auraList = JSON.parse(JSON.stringify(auraDeployMocks_1.testAuraList));
        // having a present Id triggers an update
        auraList[0].Id = '12345';
        sandboxStub.stub(deployStrategies_1.AuraDeploy.prototype, 'buildDefList').resolves(auraList);
        sandboxStub
            .stub(mockConnection.tooling, 'query')
            // @ts-ignore
            .resolves({ records: [] });
        sandboxStub.stub(mockConnection.tooling, 'create').resolves();
        const auraDeploy = new deployStrategies_1.AuraDeploy(mockConnection);
        const result = yield auraDeploy.deploy(auraDeployMocks_1.auraComponent, '');
        chai_1.expect(result).to.deep.equal({
            id: undefined,
            status: "Completed" /* Completed */,
            success: true,
            components: [
                {
                    status: client_1.ComponentStatus.Changed,
                    component: auraDeployMocks_1.auraComponent,
                    diagnostics: [],
                },
            ],
        });
    }));
    it('should set component status to "failed" when all definition files fail upsert', () => __awaiter(void 0, void 0, void 0, function* () {
        const error = Error('test error');
        sandboxStub.stub(deployStrategies_1.AuraDeploy.prototype, 'buildDefList').resolves(auraDeployMocks_1.testAuraList);
        sandboxStub
            .stub(mockConnection.tooling, 'query')
            // @ts-ignore
            .resolves({ records: [] });
        const mockToolingCreate = sandboxStub.stub(mockConnection.tooling, 'create');
        mockToolingCreate.throws(error);
        const auraDeploy = new deployStrategies_1.AuraDeploy(mockConnection);
        const result = yield auraDeploy.deploy(auraDeployMocks_1.auraComponent, '');
        chai_1.expect(result).to.deep.equal({
            id: undefined,
            status: "Failed" /* Failed */,
            success: false,
            components: [
                {
                    status: client_1.ComponentStatus.Failed,
                    component: auraDeployMocks_1.auraComponent,
                    diagnostics: auraDeployMocks_1.testAuraList.map(() => ({ message: error.message, type: 'Error' })),
                },
            ],
        });
    }));
    it('should set deploy status to "CompletedPartial" when only some definition files fail upsert', () => __awaiter(void 0, void 0, void 0, function* () {
        const error = Error('test error');
        sandboxStub.stub(deployStrategies_1.AuraDeploy.prototype, 'buildDefList').resolves(auraDeployMocks_1.testAuraList);
        sandboxStub
            .stub(mockConnection.tooling, 'query')
            // @ts-ignore
            .resolves({ records: [] });
        const mockToolingCreate = sandboxStub.stub(mockConnection.tooling, 'create');
        mockToolingCreate.resolves();
        mockToolingCreate.onSecondCall().throws(error);
        const auraDeploy = new deployStrategies_1.AuraDeploy(mockConnection);
        const result = yield auraDeploy.deploy(auraDeployMocks_1.auraComponent, '');
        chai_1.expect(result).to.deep.equal({
            id: undefined,
            status: "CompletedPartial" /* CompletedPartial */,
            success: false,
            components: [
                {
                    status: client_1.ComponentStatus.Changed,
                    component: auraDeployMocks_1.auraComponent,
                    diagnostics: [
                        {
                            message: error.message,
                            type: 'Error',
                        },
                    ],
                },
            ],
        });
    }));
});
//# sourceMappingURL=auraDeploy.test.js.map