"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
/*
 * Copyright (c) 2020, salesforce.com, inc.
 * All rights reserved.
 * Licensed under the BSD 3-Clause license.
 * For full license text, see LICENSE.txt file in the repo root or https://opensource.org/licenses/BSD-3-Clause
 */
const core_1 = require("@salesforce/core");
const chai_1 = require("chai");
const testSetup_1 = require("@salesforce/core/lib/testSetup");
const sinon_1 = require("sinon");
const metadata_registry_1 = require("../../src/metadata-registry");
const metadataApi_1 = require("../../src/client/metadataApi");
const convert_1 = require("../../src/convert");
const assert_1 = require("assert");
const fsUtil = require("../../src/utils/fileSystemHandler");
const path = require("path");
const fs = require("fs");
const i18n_1 = require("../../src/i18n");
const types_1 = require("../../src/client/types");
const stream = require("stream");
const unzipper = require("unzipper");
const os_1 = require("os");
/* eslint-disable @typescript-eslint/explicit-function-return-type */
describe('Metadata Api', () => {
    let mockConnection;
    let sandboxStub = sinon_1.createSandbox();
    const testData = new testSetup_1.MockTestOrgData();
    const registryAccess = new metadata_registry_1.RegistryAccess();
    const rootPath = path.join('file', 'path');
    const props = {
        name: 'myTestClass',
        type: metadata_registry_1.registryData.types.apexclass,
        xml: path.join(rootPath, 'myTestClass.cls-meta.xml'),
        content: path.join(rootPath, 'myTestClass.cls'),
    };
    const component = metadata_registry_1.SourceComponent.createVirtualComponent(props, [
        {
            dirPath: rootPath,
            children: [path.basename(props.xml), path.basename(props.content)],
        },
    ]);
    const retrieveResult = {
        id: '12345',
        status: types_1.RetrieveStatus.Succeeded,
        success: true,
        fileProperties: [],
        done: true,
        messages: { fileName: '', problem: '' },
        zipFile: 'thisisthezipfile',
    };
    const failedRetrieveResult = {
        id: '12345',
        status: types_1.RetrieveStatus.Failed,
        success: false,
        fileProperties: [],
        done: true,
        messages: { fileName: 'testComponent', problem: 'There was an error' },
        zipFile: 'thisisthezipfile',
    };
    const deployResult = {
        id: '12345',
        status: types_1.DeployStatus.Succeeded,
        success: true,
        details: {
            componentSuccesses: [
                // @ts-ignore
                {
                    fullName: component.fullName,
                    success: 'true',
                },
            ],
        },
    };
    const sourceDeployResult = {
        id: '12345',
        success: true,
        status: types_1.DeployStatus.Succeeded,
        components: [
            {
                component,
                status: types_1.ComponentStatus.Unchanged,
                diagnostics: [],
            },
        ],
    };
    const testingBuffer = Buffer.from('testingBuffer');
    const deployPath = path.join('file', 'path', 'myTestClass.cls');
    const outputDir = path.join('path', 'to', 'output', 'dir');
    let metadataClient;
    let registryStub = sandboxStub.stub();
    let convertStub = sandboxStub.stub();
    let deployIdStub = sandboxStub.stub();
    beforeEach(() => __awaiter(void 0, void 0, void 0, function* () {
        sandboxStub = sinon_1.createSandbox();
        mockConnection = yield core_1.Connection.create({
            authInfo: yield core_1.AuthInfo.create({
                username: testData.username,
            }),
        });
        metadataClient = new metadataApi_1.MetadataApi(mockConnection, registryAccess);
        registryStub = sandboxStub.stub(registryAccess, 'getComponentsFromPath').returns([component]);
        convertStub = sandboxStub
            .stub(convert_1.MetadataConverter.prototype, 'convert')
            .withArgs(sinon_1.match.any, 'metadata', { type: 'zip' })
            .resolves({
            zipBuffer: testingBuffer,
        });
    }));
    afterEach(() => {
        sandboxStub.restore();
    });
    it('Should check that the default options are correct', () => __awaiter(void 0, void 0, void 0, function* () {
        const defaultOptions = {
            rollbackOnError: true,
            ignoreWarnings: false,
            checkOnly: false,
            singlePackage: true,
        };
        chai_1.expect(metadataApi_1.DEFAULT_API_OPTIONS).to.deep.equal(defaultOptions);
    }));
    it('Should correctly deploy metadata components from paths', () => __awaiter(void 0, void 0, void 0, function* () {
        // @ts-ignore minimum info required
        deployIdStub = sandboxStub.stub(mockConnection.metadata, 'deploy').resolves({
            id: '12345',
        });
        const deployPollStub = sandboxStub
            .stub(mockConnection.metadata, 'checkDeployStatus')
            .withArgs('12345', true)
            // @ts-ignore minimum info required
            .resolves(deployResult);
        const deploys = yield metadataClient.deployWithPaths(deployPath);
        chai_1.expect(registryStub.calledImmediatelyBefore(convertStub)).to.be.true;
        chai_1.expect(convertStub.calledImmediatelyBefore(deployIdStub)).to.be.true;
        chai_1.expect(deployIdStub.calledImmediatelyBefore(deployPollStub)).to.be.true;
        chai_1.expect(deploys).to.deep.equal(sourceDeployResult);
    }));
    it('Should correctly deploy metadata components with custom deploy options', () => __awaiter(void 0, void 0, void 0, function* () {
        const apiOptions = {
            allowMissingFiles: true,
            autoUpdatePackage: true,
            checkOnly: true,
            ignoreWarnings: true,
            performRetrieve: true,
            purgeOnDelete: true,
            rollbackOnError: true,
            runAllTests: true,
            runTests: ['test1', 'test2'],
            singlePackage: true,
        };
        deployIdStub = sandboxStub
            .stub(mockConnection.metadata, 'deploy')
            .withArgs(testingBuffer, apiOptions)
            // @ts-ignore
            .resolves({
            id: '12345',
        });
        // @ts-ignore
        sandboxStub.stub(mockConnection.metadata, 'checkDeployStatus').resolves(deployResult);
        yield metadataClient.deployWithPaths(deployPath, { apiOptions });
        chai_1.expect(deployIdStub.args).to.deep.equal([[testingBuffer, apiOptions]]);
    }));
    it('Should correctly deploy metadata components with default deploy options', () => __awaiter(void 0, void 0, void 0, function* () {
        deployIdStub = sandboxStub
            .stub(mockConnection.metadata, 'deploy')
            .withArgs(testingBuffer, metadataApi_1.DEFAULT_API_OPTIONS)
            // @ts-ignore
            .resolves({
            id: '12345',
        });
        // @ts-ignore
        sandboxStub.stub(mockConnection.metadata, 'checkDeployStatus').resolves(deployResult);
        yield metadataClient.deployWithPaths(deployPath);
        chai_1.expect(deployIdStub.args).to.deep.equal([[testingBuffer, metadataApi_1.DEFAULT_API_OPTIONS]]);
    }));
    it('Should correctly deploy metadata components with default and custom deploy options', () => __awaiter(void 0, void 0, void 0, function* () {
        const apiOptions = {
            rollbackOnError: true,
            ignoreWarnings: false,
            checkOnly: true,
            autoUpdatePackage: true,
            singlePackage: false,
        };
        deployIdStub = sandboxStub
            .stub(mockConnection.metadata, 'deploy')
            .withArgs(testingBuffer, apiOptions)
            // @ts-ignore
            .resolves({
            id: '12345',
        });
        // @ts-ignore
        sandboxStub.stub(mockConnection.metadata, 'checkDeployStatus').resolves(deployResult);
        yield metadataClient.deployWithPaths(deployPath, {
            apiOptions: { checkOnly: true, autoUpdatePackage: true, singlePackage: false },
        });
        chai_1.expect(deployIdStub.args).to.deep.equal([[testingBuffer, apiOptions]]);
    }));
    describe('Metadata Status Poll', () => {
        it('should verify successful status poll', () => __awaiter(void 0, void 0, void 0, function* () {
            // @ts-ignore
            deployIdStub = sandboxStub.stub(mockConnection.metadata, 'deploy').resolves({
                id: '12345',
            });
            sandboxStub
                .stub(mockConnection.metadata, 'checkDeployStatus')
                // @ts-ignore
                .resolves(deployResult);
            const deploys = yield metadataClient.deployWithPaths(deployPath);
            chai_1.expect(deploys).to.deep.equal(sourceDeployResult);
        }));
        it('should throw correct error for unexpected issue', () => __awaiter(void 0, void 0, void 0, function* () {
            // @ts-ignore
            deployIdStub = sandboxStub.stub(mockConnection.metadata, 'deploy').resolves({
                id: '12345',
            });
            sandboxStub.stub(mockConnection.metadata, 'checkDeployStatus').throws('unexpected error');
            try {
                yield metadataClient.deployWithPaths(deployPath);
                assert_1.fail('request should have failed');
            }
            catch (e) {
                chai_1.expect(e.message).contains(i18n_1.nls.localize('md_request_fail', 'unexpected error'));
            }
        }));
        it('should verify timeout status poll', () => __awaiter(void 0, void 0, void 0, function* () {
            // @ts-ignore
            deployIdStub = sandboxStub.stub(mockConnection.metadata, 'deploy').resolves({
                id: '12345',
            });
            const deployOptionWait = {
                wait: 100,
            };
            const deployPollStub = sandboxStub.stub(mockConnection.metadata, 'checkDeployStatus');
            // @ts-ignore
            deployPollStub.resolves({ status: 'Pending', success: false });
            const result = yield metadataClient.deployWithPaths(deployPath, deployOptionWait);
            chai_1.expect(result.status).to.equal('Pending');
            chai_1.expect(result.success).to.be.false;
        }));
    });
    describe('Metadata Deploy Result', () => {
        it('should set deploy operation status correctly', () => __awaiter(void 0, void 0, void 0, function* () {
            // @ts-ignore
            sandboxStub.stub(mockConnection.metadata, 'deploy').resolves({
                id: '12345',
            });
            sandboxStub
                .stub(mockConnection.metadata, 'checkDeployStatus')
                // @ts-ignore minimum info required
                .resolves({
                success: true,
                id: '1234',
                status: 'Succeeded',
            });
            const result = yield metadataClient.deploy(component);
            chai_1.expect(result).to.deep.equal({
                id: '1234',
                success: true,
                status: 'Succeeded',
            });
        }));
        it('should set Changed component status for changed component', () => __awaiter(void 0, void 0, void 0, function* () {
            // @ts-ignore
            sandboxStub.stub(mockConnection.metadata, 'deploy').resolves({
                id: '12345',
            });
            sandboxStub
                .stub(mockConnection.metadata, 'checkDeployStatus')
                // @ts-ignore minimum info required
                .resolves({
                success: true,
                id: '1234',
                status: 'Succeeded',
                details: {
                    // @ts-ignore
                    componentSuccesses: [
                        {
                            changed: 'true',
                            created: 'false',
                            deleted: 'false',
                            fullName: component.fullName,
                            componentType: component.type.name,
                        },
                    ],
                },
            });
            const result = yield metadataClient.deploy(component);
            chai_1.expect(result.components).to.deep.equal([
                {
                    component,
                    status: types_1.ComponentStatus.Changed,
                    diagnostics: [],
                },
            ]);
        }));
        it('should set Created component status for created component', () => __awaiter(void 0, void 0, void 0, function* () {
            // @ts-ignore
            sandboxStub.stub(mockConnection.metadata, 'deploy').resolves({
                id: '12345',
            });
            sandboxStub
                .stub(mockConnection.metadata, 'checkDeployStatus')
                // @ts-ignore minimum info required
                .resolves({
                success: true,
                id: '1234',
                status: 'Succeeded',
                details: {
                    // @ts-ignore
                    componentSuccesses: [
                        {
                            changed: 'false',
                            created: 'true',
                            deleted: 'false',
                            fullName: component.fullName,
                            componentType: component.type.name,
                        },
                    ],
                },
            });
            const result = yield metadataClient.deploy(component);
            chai_1.expect(result.components).to.deep.equal([
                {
                    component,
                    status: types_1.ComponentStatus.Created,
                    diagnostics: [],
                },
            ]);
        }));
        it('should set Deleted component status for deleted component', () => __awaiter(void 0, void 0, void 0, function* () {
            // @ts-ignore
            sandboxStub.stub(mockConnection.metadata, 'deploy').resolves({
                id: '12345',
            });
            sandboxStub
                .stub(mockConnection.metadata, 'checkDeployStatus')
                // @ts-ignore minimum info required
                .resolves({
                success: true,
                id: '1234',
                status: 'Succeeded',
                details: {
                    // @ts-ignore
                    componentSuccesses: {
                        changed: 'false',
                        created: 'false',
                        deleted: 'true',
                        fullName: component.fullName,
                        componentType: component.type.name,
                    },
                },
            });
            const result = yield metadataClient.deploy(component);
            chai_1.expect(result.components).to.deep.equal([
                {
                    component,
                    status: types_1.ComponentStatus.Deleted,
                    diagnostics: [],
                },
            ]);
        }));
        it('should set Failed component status for failed component', () => __awaiter(void 0, void 0, void 0, function* () {
            // @ts-ignore
            sandboxStub.stub(mockConnection.metadata, 'deploy').resolves({
                id: '12345',
            });
            sandboxStub
                .stub(mockConnection.metadata, 'checkDeployStatus')
                // @ts-ignore minimum info required
                .resolves({
                success: true,
                id: '1234',
                status: 'Failed',
                details: {
                    // @ts-ignore
                    componentFailures: {
                        success: 'false',
                        changed: 'false',
                        created: 'false',
                        deleted: 'false',
                        fullName: component.fullName,
                        componentType: component.type.name,
                    },
                },
            });
            const result = yield metadataClient.deploy(component);
            chai_1.expect(result.components).to.deep.equal([
                {
                    component,
                    status: types_1.ComponentStatus.Failed,
                    diagnostics: [],
                },
            ]);
        }));
        it('should aggregate diagnostics for a component', () => __awaiter(void 0, void 0, void 0, function* () {
            // @ts-ignore
            sandboxStub.stub(mockConnection.metadata, 'deploy').resolves({
                id: '12345',
            });
            sandboxStub
                .stub(mockConnection.metadata, 'checkDeployStatus')
                // @ts-ignore minimum info required
                .resolves({
                success: false,
                id: '1234',
                status: 'Failed',
                details: {
                    // @ts-ignore
                    componentFailures: [
                        {
                            success: 'false',
                            changed: 'false',
                            created: 'false',
                            deleted: 'false',
                            fullName: component.fullName,
                            componentType: component.type.name,
                            problem: 'Expected ;',
                            problemType: 'Error',
                            lineNumber: 3,
                            columnNumber: 7,
                        },
                        {
                            success: 'false',
                            changed: 'false',
                            created: 'false',
                            deleted: 'false',
                            fullName: component.fullName,
                            componentType: component.type.name,
                            problem: 'Symbol test does not exist',
                            problemType: 'Error',
                            lineNumber: 8,
                            columnNumber: 23,
                        },
                    ],
                },
            });
            const result = yield metadataClient.deploy(component);
            chai_1.expect(result.components).to.deep.equal([
                {
                    component,
                    status: types_1.ComponentStatus.Failed,
                    diagnostics: [
                        {
                            lineNumber: 3,
                            columnNumber: 7,
                            message: 'Expected ;',
                            type: 'Error',
                        },
                        {
                            lineNumber: 8,
                            columnNumber: 23,
                            message: 'Symbol test does not exist',
                            type: 'Error',
                        },
                    ],
                },
            ]);
        }));
        it('should fix lwc deploy message issue', () => __awaiter(void 0, void 0, void 0, function* () {
            const bundlePath = path.join('path', 'to', 'lwc', 'test');
            const props = {
                name: 'test',
                type: metadata_registry_1.registryData.types.lightningcomponentbundle,
                xml: path.join(bundlePath, 'test.js-meta.xml'),
                content: bundlePath,
            };
            const component = metadata_registry_1.SourceComponent.createVirtualComponent(props, [
                {
                    dirPath: bundlePath,
                    children: [path.basename(props.xml), 'test.js', 'test.html'],
                },
            ]);
            // @ts-ignore
            sandboxStub.stub(mockConnection.metadata, 'deploy').resolves({
                id: '12345',
            });
            sandboxStub
                .stub(mockConnection.metadata, 'checkDeployStatus')
                // @ts-ignore minimum info required
                .resolves({
                success: true,
                id: '1234',
                status: 'Succeeded',
                details: {
                    // @ts-ignore
                    componentSuccesses: [
                        {
                            success: 'false',
                            changed: 'false',
                            created: 'true',
                            deleted: 'false',
                            fullName: 'markup://c:test',
                            componentType: component.type.name,
                        },
                    ],
                },
            });
            const result = yield metadataClient.deploy(component);
            chai_1.expect(result.components).to.deep.equal([
                {
                    component,
                    status: types_1.ComponentStatus.Created,
                    diagnostics: [],
                },
            ]);
        }));
    });
    describe('Metadata API Retrieve', () => __awaiter(void 0, void 0, void 0, function* () {
        let retrieveStub;
        let extractStub;
        let removeStub;
        let retrieveStatusStub;
        beforeEach(() => __awaiter(void 0, void 0, void 0, function* () {
            const mockRetrieve = new stream.PassThrough();
            retrieveStub = sandboxStub
                .stub(mockConnection.metadata, 'retrieve')
                // @ts-ignore
                .resolves({ id: 'xxxxxxx' });
            // @ts-ignore
            sandboxStub.stub(fs, 'createReadStream').returns(mockRetrieve);
            const mockStream = new stream.PassThrough();
            sandboxStub.stub(stream, 'pipeline').resolves();
            // @ts-ignore
            extractStub = sandboxStub.stub(unzipper, 'Extract').returns(mockStream);
            setTimeout(() => {
                mockStream.emit('close');
            }, 100);
            retrieveStatusStub = sandboxStub
                .stub(mockConnection.metadata, 'checkRetrieveStatus')
                // @ts-ignore
                .resolves(retrieveResult);
            registryStub.onFirstCall().returns([component]);
            registryStub.onSecondCall().returns([component]);
            convertStub
                .withArgs(sinon_1.match.any, 'source', { type: 'directory', outputDirectory: outputDir })
                .resolves({
                packagePath: outputDir,
            });
            removeStub = sandboxStub.stub(fsUtil, 'deleteDirectory');
        }));
        afterEach(() => {
            sandboxStub.restore();
        });
        it('should correctly format retrieve request', () => __awaiter(void 0, void 0, void 0, function* () {
            const apiVersion = registryAccess.getApiVersion();
            const options = {
                components: [component],
                output: outputDir,
            };
            const retrieveRequest = {
                apiVersion,
                unpackaged: {
                    types: { members: 'myTestClass', name: 'ApexClass' },
                    version: parseInt(apiVersion),
                },
            };
            yield metadataClient.retrieve(options);
            chai_1.expect(retrieveStub.calledWith(retrieveRequest)).to.be.true;
        }));
        it('should correctly retrieve components and extract to directory', () => __awaiter(void 0, void 0, void 0, function* () {
            const tempDir = path.join(os_1.tmpdir(), '.sfdx', 'tmp');
            const options = {
                components: [component],
                output: outputDir,
            };
            yield metadataClient.retrieve(options);
            chai_1.expect(extractStub.calledOnce).to.be.true;
            chai_1.expect(extractStub.calledWith({ path: tempDir })).to.be.true;
        }));
        it('should correctly retrieve components and clean temp folder', () => __awaiter(void 0, void 0, void 0, function* () {
            const tempDir = path.join(os_1.tmpdir(), '.sfdx', 'tmp');
            const options = {
                components: [component],
                output: outputDir,
            };
            yield metadataClient.retrieve(options);
            chai_1.expect(removeStub.called).to.be.true;
            chai_1.expect(removeStub.calledWith(tempDir)).to.be.true;
        }));
        it('should convert the retrieved components', () => __awaiter(void 0, void 0, void 0, function* () {
            const options = {
                components: [component],
                output: outputDir,
            };
            yield metadataClient.retrieve(options);
            chai_1.expect(convertStub.withArgs(sinon_1.match.any, 'source', {
                type: 'directory',
                outputDirectory: outputDir,
            }).calledOnce).to.be.true;
        }));
        it('should return successful result in SourceRetrieveResult format', () => __awaiter(void 0, void 0, void 0, function* () {
            const options = {
                components: [component],
                output: outputDir,
            };
            const sourceRetrieveResult = {
                success: true,
                components: [component],
                id: '12345',
                message: {
                    fileName: '',
                    problem: '',
                },
                status: types_1.RetrieveStatus.Succeeded,
            };
            const result = yield metadataClient.retrieve(options);
            chai_1.expect(result).to.eql(sourceRetrieveResult);
        }));
        it('should return failed result in SourceRetrieveResult format', () => __awaiter(void 0, void 0, void 0, function* () {
            retrieveStatusStub.resolves(failedRetrieveResult);
            const options = {
                components: [component],
                output: outputDir,
            };
            const sourceRetrieveResult = {
                success: false,
                components: [],
                id: '12345',
                message: { fileName: 'testComponent', problem: 'There was an error' },
                status: types_1.RetrieveStatus.Failed,
            };
            const result = yield metadataClient.retrieve(options);
            chai_1.expect(result).to.eql(sourceRetrieveResult);
        }));
        it('should throw an error if theres an error during retrieve or conversion', () => __awaiter(void 0, void 0, void 0, function* () {
            const errorMsg = 'test error';
            const error = new Error(errorMsg);
            const options = {
                components: [component],
                output: outputDir,
            };
            retrieveStub.throws(error);
            try {
                yield metadataClient.retrieve(options);
                chai_1.expect.fail('Should have failed');
            }
            catch (e) {
                chai_1.expect(e.message).to.equal(errorMsg);
            }
        }));
        it('should only retrieve unique components when retrieving with paths', () => __awaiter(void 0, void 0, void 0, function* () {
            const rootPath = path.join('file', 'path');
            const contentPaths = [
                path.join(rootPath, 'myTestClass1.cls'),
                path.join(rootPath, 'myTestClass2.cls'),
                path.join(rootPath, 'myTestClass3.cls'),
            ];
            const xmlPaths = [
                path.join(rootPath, 'myTestClass1.cls-meta.xml'),
                path.join(rootPath, 'myTestClass2.cls-meta.xml'),
                path.join(rootPath, 'myTestClass3.cls-meta.xml'),
            ];
            const firstProp = {
                name: 'myTestClass1',
                type: metadata_registry_1.registryData.types.apexclass,
                xml: xmlPaths[0],
                content: contentPaths[0],
            };
            const secondProp = {
                name: 'myTestClass2',
                type: metadata_registry_1.registryData.types.apexclass,
                xml: xmlPaths[1],
                content: contentPaths[1],
            };
            const thirdProp = {
                name: 'myTestClass2',
                type: metadata_registry_1.registryData.types.apexclass,
                xml: xmlPaths[1],
                content: contentPaths[1],
            };
            const firstComponent = metadata_registry_1.SourceComponent.createVirtualComponent(firstProp, [
                {
                    dirPath: rootPath,
                    children: [path.basename(firstProp.xml), path.basename(firstProp.content)],
                },
            ]);
            const secondComponent = metadata_registry_1.SourceComponent.createVirtualComponent(secondProp, [
                {
                    dirPath: rootPath,
                    children: [path.basename(secondProp.xml), path.basename(secondProp.content)],
                },
            ]);
            const thirdComponent = metadata_registry_1.SourceComponent.createVirtualComponent(thirdProp, [
                {
                    dirPath: rootPath,
                    children: [path.basename(thirdProp.xml), path.basename(thirdProp.content)],
                },
            ]);
            const wait = 1000;
            const options = {
                paths: contentPaths,
                output: outputDir,
                wait,
            };
            const mdRetrieveStub = sandboxStub.stub(metadataClient, 'retrieve');
            registryStub.onFirstCall().returns([firstComponent, secondComponent, thirdComponent]);
            registryStub.onSecondCall().returns([firstComponent, secondComponent, thirdComponent]);
            registryStub.onThirdCall().returns([firstComponent, secondComponent, thirdComponent]);
            yield metadataClient.retrieveWithPaths(options);
            chai_1.expect(mdRetrieveStub.calledWith({
                components: [firstComponent, secondComponent, thirdComponent],
                namespace: options.namespace,
                output: options.output,
                wait,
            }));
        }));
    }));
});
//# sourceMappingURL=metadataApi.test.js.map