"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
/*
 * Copyright (c) 2020, salesforce.com, inc.
 * All rights reserved.
 * Licensed under the BSD 3-Clause license.
 * For full license text, see LICENSE.txt file in the repo root or https://opensource.org/licenses/BSD-3-Clause
 */
const convertTransaction_1 = require("../../src/convert/convertTransaction");
const registry_1 = require("../mock/registry");
const chai_1 = require("chai");
const sinon_1 = require("sinon");
const finalizers_1 = require("../mock/convert/finalizers");
const path_1 = require("path");
const fs = require("fs");
const constants_1 = require("../../src/utils/constants");
const streams_1 = require("../../src/convert/streams");
const env = sinon_1.createSandbox();
describe('Convert Transaction Constructs', () => {
    afterEach(() => env.restore());
    describe('ConvertTransaction', () => {
        it('should only set a finalizer once per transaction', () => {
            const transaction = new convertTransaction_1.ConvertTransaction();
            // @ts-ignore visible for testing
            const setSpy = env.spy(transaction.finalizers, 'set');
            transaction.addFinalizer(finalizers_1.TestFinalizerNoWrites);
            transaction.addFinalizer(finalizers_1.TestFinalizerNoWrites);
            chai_1.expect(setSpy.calledOnce).to.be.true;
            env.restore();
        });
        it('should yield results of finalizers upon executeFinalizers', () => {
            const transaction = new convertTransaction_1.ConvertTransaction();
            transaction.addFinalizer(finalizers_1.TestFinalizerNoWrites);
            transaction.addFinalizer(finalizers_1.TestFinalizerNoResult);
            const results = Array.from(transaction.executeFinalizers());
            chai_1.expect(results).to.deep.equal([
                {
                    component: registry_1.keanu.KEANU_COMPONENT,
                    writeInfos: [],
                },
                undefined,
            ]);
        });
    });
    describe('RecompositionFinalizer', () => {
        it('should return a WriterFormat with recomposed data', () => {
            const component = registry_1.regina.REGINA_COMPONENT;
            const children = component.getChildren();
            const readStub = env.stub(fs, 'readFileSync');
            readStub.withArgs(children[0].xml).returns('<Y><test>child1</test></Y>');
            readStub.withArgs(children[1].xml).returns('<X><test>child2</test></X>');
            const finalizer = new convertTransaction_1.RecompositionFinalizer();
            const state = {
                recompose: {
                    Test__c: {
                        component,
                        children,
                    },
                },
            };
            const result = finalizer.finalize(state);
            chai_1.expect(result).to.deep.equal([
                {
                    component: component,
                    writeInfos: [
                        {
                            source: new streams_1.JsToXml({
                                ReginaKing: {
                                    [constants_1.XML_NS_KEY]: constants_1.XML_NS,
                                    ys: [{ test: 'child1' }],
                                    xs: [{ test: 'child2' }],
                                },
                            }),
                            relativeDestination: path_1.join('reginas', 'a.regina'),
                        },
                    ],
                },
            ]);
        });
    });
});
//# sourceMappingURL=convertTransaction.test.js.map