"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
/*
 * Copyright (c) 2020, salesforce.com, inc.
 * All rights reserved.
 * Licensed under the BSD 3-Clause license.
 * For full license text, see LICENSE.txt file in the repo root or https://opensource.org/licenses/BSD-3-Clause
 */
const registry_1 = require("../../mock/registry");
const defaultMetadataTransformer_1 = require("../../../src/convert/transformers/defaultMetadataTransformer");
const path_1 = require("path");
const sinon_1 = require("sinon");
const readables_1 = require("../../mock/convert/readables");
const fs = require("fs");
const chai_1 = require("chai");
const utils_1 = require("../../../src/utils");
const env = sinon_1.createSandbox();
describe('DefaultMetadataTransformer', () => {
    beforeEach(() => 
    // @ts-ignore mock readable isn't an fs readable specifically
    env.stub(fs, 'createReadStream').callsFake((fsPath) => new readables_1.TestReadable(fsPath)));
    afterEach(() => env.restore());
    describe('toMetadataFormat', () => {
        it('should create a WriteInfo for each file in the component', () => {
            const component = registry_1.simon.SIMON_COMPONENT;
            const transformer = new defaultMetadataTransformer_1.DefaultMetadataTransformer(registry_1.mockRegistry);
            const { directoryName } = component.type;
            const relativeBundle = path_1.join(directoryName, path_1.basename(registry_1.simon.SIMON_BUNDLE_PATH));
            const expectedInfos = [];
            for (const source of component.walkContent()) {
                expectedInfos.push({
                    relativeDestination: path_1.join(relativeBundle, path_1.basename(source)),
                    source: fs.createReadStream(source),
                });
            }
            expectedInfos.push({
                relativeDestination: path_1.join(relativeBundle, registry_1.simon.SIMON_XML_NAME),
                source: fs.createReadStream(component.xml),
            });
            chai_1.expect(transformer.toMetadataFormat(component)).to.deep.equal({
                component,
                writeInfos: expectedInfos,
            });
        });
        it('should strip the -meta.xml suffix for components with no content', () => {
            const component = registry_1.gene.GENE_COMPONENT;
            const transformer = new defaultMetadataTransformer_1.DefaultMetadataTransformer(registry_1.mockRegistry);
            const { directoryName } = component.type;
            const fileName = `${component.fullName}.${component.type.suffix}`;
            const expectedInfos = [
                {
                    relativeDestination: path_1.join(directoryName, fileName),
                    source: fs.createReadStream(component.xml),
                },
            ];
            chai_1.expect(transformer.toMetadataFormat(component)).to.deep.equal({
                component,
                writeInfos: expectedInfos,
            });
        });
        it('should handle folder type components with no content', () => {
            const component = registry_1.kathy.KATHY_COMPONENTS[0];
            const fullNameParts = component.fullName.split('/');
            const { directoryName } = component.type;
            const transformer = new defaultMetadataTransformer_1.DefaultMetadataTransformer(registry_1.mockRegistry);
            const expectedInfos = [
                {
                    relativeDestination: path_1.join(directoryName, fullNameParts[0], `${fullNameParts[1]}.${component.type.suffix}`),
                    source: fs.createReadStream(component.xml),
                },
            ];
            chai_1.expect(transformer.toMetadataFormat(component)).to.deep.equal({
                component,
                writeInfos: expectedInfos,
            });
        });
    });
    describe('toSourceFormat', () => {
        it('should create a WriteInfo for each file in the component', () => {
            const component = registry_1.simon.SIMON_COMPONENT;
            const transformer = new defaultMetadataTransformer_1.DefaultMetadataTransformer(registry_1.mockRegistry);
            const { directoryName } = component.type;
            const relativeBundle = path_1.join(directoryName, path_1.basename(registry_1.simon.SIMON_BUNDLE_PATH));
            const expectedInfos = [];
            for (const source of component.walkContent()) {
                expectedInfos.push({
                    relativeDestination: path_1.join(relativeBundle, path_1.basename(source)),
                    source: fs.createReadStream(source),
                });
            }
            expectedInfos.push({
                relativeDestination: path_1.join(relativeBundle, registry_1.simon.SIMON_XML_NAME),
                source: fs.createReadStream(component.xml),
            });
            chai_1.expect(transformer.toSourceFormat(component)).to.deep.equal({
                component,
                writeInfos: expectedInfos,
            });
        });
        it('should add in the -meta.xml suffix for components with no content', () => {
            const component = registry_1.gene.GENE_MD_FORMAT_COMPONENT;
            const transformer = new defaultMetadataTransformer_1.DefaultMetadataTransformer(registry_1.mockRegistry);
            const { directoryName } = component.type;
            const fileName = `${component.fullName}.${component.type.suffix}${utils_1.META_XML_SUFFIX}`;
            const expectedInfos = [
                {
                    relativeDestination: path_1.join(directoryName, fileName),
                    source: fs.createReadStream(component.xml),
                },
            ];
            chai_1.expect(transformer.toSourceFormat(component)).to.deep.equal({
                component,
                writeInfos: expectedInfos,
            });
        });
        it('should handle folder type components with no content', () => {
            const component = registry_1.kathy.KATHY_MD_FORMAT_COMPONENTS[0];
            const fullNameParts = component.fullName.split('/');
            const { directoryName } = component.type;
            const transformer = new defaultMetadataTransformer_1.DefaultMetadataTransformer(registry_1.mockRegistry);
            const expectedInfos = [
                {
                    relativeDestination: path_1.join(directoryName, fullNameParts[0], `${fullNameParts[1]}.${component.type.suffix}${utils_1.META_XML_SUFFIX}`),
                    source: fs.createReadStream(component.xml),
                },
            ];
            chai_1.expect(transformer.toSourceFormat(component)).to.deep.equal({
                component,
                writeInfos: expectedInfos,
            });
        });
    });
});
//# sourceMappingURL=defaultMetadataTransformer.js.map