"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
/*
 * Copyright (c) 2020, salesforce.com, inc.
 * All rights reserved.
 * Licensed under the BSD 3-Clause license.
 * For full license text, see LICENSE.txt file in the repo root or https://opensource.org/licenses/BSD-3-Clause
 */
const staticResourceMetadataTransformer_1 = require("../../../src/convert/transformers/staticResourceMetadataTransformer");
const sinon_1 = require("sinon");
const path_1 = require("path");
const mixedContentSingleFileConstants_1 = require("../../mock/registry/mixedContentSingleFileConstants");
const utils_1 = require("../../../src/utils");
const fs = require("fs");
const chai_1 = require("chai");
const readables_1 = require("../../mock/convert/readables");
const tarajiConstants_1 = require("../../mock/registry/tarajiConstants");
const constants_1 = require("../../../src/utils/constants");
const archiver = require("archiver");
const src_1 = require("../../../src");
const errors_1 = require("../../../src/errors");
const i18n_1 = require("../../../src/i18n");
const unzipper_1 = require("unzipper");
const registry_1 = require("../../mock/registry");
const env = sinon_1.createSandbox();
describe('StaticResourceMetadataTransformer', () => {
    beforeEach(() => 
    // @ts-ignore mock readable isn't an fs readable specifically
    env.stub(fs, 'createReadStream').callsFake((fsPath) => new readables_1.TestReadable(fsPath)));
    afterEach(() => env.restore());
    describe('toMetadataFormat', () => {
        it('should rename extension to .resource for content file', () => {
            const component = mixedContentSingleFileConstants_1.MC_SINGLE_FILE_COMPONENT;
            const { type, content, xml } = component;
            env.stub(component, 'parseXml').returns({
                StaticResource: {
                    contentType: 'png',
                },
            });
            const transformer = new staticResourceMetadataTransformer_1.StaticResourceMetadataTransformer(registry_1.mockRegistry);
            const expectedInfos = [
                {
                    source: fs.createReadStream(content),
                    relativeDestination: path_1.join(type.directoryName, `${utils_1.baseName(content)}.${type.suffix}`),
                },
                {
                    source: fs.createReadStream(xml),
                    relativeDestination: path_1.join(type.directoryName, path_1.basename(xml)),
                },
            ];
            chai_1.expect(transformer.toMetadataFormat(component)).to.deep.equal({
                component,
                writeInfos: expectedInfos,
            });
        });
        it('should zip directory content for all supported archive mime types', () => {
            const component = src_1.SourceComponent.createVirtualComponent(tarajiConstants_1.TARAJI_COMPONENT, tarajiConstants_1.TARAJI_VIRTUAL_FS);
            const { type, content, xml } = component;
            const archive = archiver.create('zip', { zlib: { level: 3 } });
            const archiveDirStub = env.stub(archive, 'directory');
            const archiveFinalizeStub = env.stub(archive, 'finalize');
            const parseXmlStub = env.stub(component, 'parseXml');
            env.stub(archiver, 'create').returns(archive);
            const transformer = new staticResourceMetadataTransformer_1.StaticResourceMetadataTransformer(registry_1.mockRegistry);
            const expectedInfos = [
                {
                    source: archive,
                    relativeDestination: path_1.join(type.directoryName, `${utils_1.baseName(content)}.${type.suffix}`),
                },
                {
                    source: fs.createReadStream(xml),
                    relativeDestination: path_1.join(type.directoryName, path_1.basename(xml)),
                },
            ];
            for (const contentType of constants_1.ARCHIVE_MIME_TYPES) {
                parseXmlStub.returns({ StaticResource: { contentType } });
                chai_1.expect(transformer.toMetadataFormat(component)).to.deep.equal({
                    component,
                    writeInfos: expectedInfos,
                });
                chai_1.expect(archiveDirStub.calledOnceWith(content, false)).to.be.true;
                chai_1.expect(archiveFinalizeStub.calledImmediatelyAfter(archiveDirStub)).to.be.true;
                archiveDirStub.resetHistory();
            }
        });
        it('should throw an error if content is directory but contentType is not an archive type', () => {
            const component = src_1.SourceComponent.createVirtualComponent(tarajiConstants_1.TARAJI_COMPONENT, tarajiConstants_1.TARAJI_VIRTUAL_FS);
            const contentType = 'nonArchiveType';
            env.stub(component, 'parseXml').returns({ StaticResource: { contentType } });
            const transformer = new staticResourceMetadataTransformer_1.StaticResourceMetadataTransformer(registry_1.mockRegistry);
            chai_1.assert.throws(() => transformer.toMetadataFormat(component), errors_1.LibraryError, i18n_1.nls.localize('error_static_resource_expected_archive_type', [contentType, component.name]));
        });
    });
    describe('toSourceFormat', () => {
        it('should rename extension from .resource to a mime extension for content file', () => {
            const component = mixedContentSingleFileConstants_1.MC_SINGLE_FILE_COMPONENT;
            const { type, content, xml } = component;
            env.stub(component, 'parseXml').returns({
                StaticResource: {
                    contentType: 'image/png',
                },
            });
            const transformer = new staticResourceMetadataTransformer_1.StaticResourceMetadataTransformer(registry_1.mockRegistry);
            const expectedInfos = [
                {
                    source: fs.createReadStream(content),
                    relativeDestination: path_1.join(type.directoryName, `${utils_1.baseName(content)}.png`),
                },
                {
                    source: fs.createReadStream(xml),
                    relativeDestination: path_1.join(type.directoryName, path_1.basename(xml)),
                },
            ];
            chai_1.expect(transformer.toSourceFormat(component)).to.deep.equal({
                component,
                writeInfos: expectedInfos,
            });
        });
        it('should rename extension from .resource for a fallback mime extension', () => {
            const component = mixedContentSingleFileConstants_1.MC_SINGLE_FILE_COMPONENT;
            const { type, content, xml } = component;
            env.stub(component, 'parseXml').returns({
                StaticResource: {
                    contentType: 'application/x-javascript',
                },
            });
            const transformer = new staticResourceMetadataTransformer_1.StaticResourceMetadataTransformer(registry_1.mockRegistry);
            const expectedInfos = [
                {
                    source: fs.createReadStream(content),
                    relativeDestination: path_1.join(type.directoryName, `${utils_1.baseName(content)}.js`),
                },
                {
                    source: fs.createReadStream(xml),
                    relativeDestination: path_1.join(type.directoryName, path_1.basename(xml)),
                },
            ];
            chai_1.expect(transformer.toSourceFormat(component)).to.deep.equal({
                component,
                writeInfos: expectedInfos,
            });
        });
        it('should rename extension from .resource for an unsupported mime extension', () => {
            const component = mixedContentSingleFileConstants_1.MC_SINGLE_FILE_COMPONENT;
            const { type, content, xml } = component;
            env.stub(component, 'parseXml').returns({
                StaticResource: {
                    contentType: 'application/x-myspace',
                },
            });
            const transformer = new staticResourceMetadataTransformer_1.StaticResourceMetadataTransformer(registry_1.mockRegistry);
            const expectedInfos = [
                {
                    source: fs.createReadStream(content),
                    relativeDestination: path_1.join(type.directoryName, `${utils_1.baseName(content)}.bin`),
                },
                {
                    source: fs.createReadStream(xml),
                    relativeDestination: path_1.join(type.directoryName, path_1.basename(xml)),
                },
            ];
            chai_1.expect(transformer.toSourceFormat(component)).to.deep.equal({
                component,
                writeInfos: expectedInfos,
            });
        });
        it('should ignore components without content', () => {
            const component = Object.assign({}, mixedContentSingleFileConstants_1.MC_SINGLE_FILE_COMPONENT);
            component.content = undefined;
            const transformer = new staticResourceMetadataTransformer_1.StaticResourceMetadataTransformer(registry_1.mockRegistry);
            chai_1.expect(transformer.toSourceFormat(component)).to.deep.equal({
                component,
                writeInfos: [],
            });
        });
        it('should extract an archive', () => __awaiter(void 0, void 0, void 0, function* () {
            const component = mixedContentSingleFileConstants_1.MC_SINGLE_FILE_COMPONENT;
            const { type, xml } = component;
            env.stub(component, 'parseXml').returns({
                StaticResource: {
                    contentType: 'application/zip',
                },
            });
            const cd = {
                files: [
                    {
                        path: 'a',
                        type: 'Directory',
                        stream: () => {
                            return null;
                        },
                    },
                    {
                        path: 'b/c.css',
                        type: 'File',
                        stream: () => {
                            return null;
                        },
                    },
                ],
            };
            env.stub(unzipper_1.Open, 'file').resolves(cd);
            const transformer = new staticResourceMetadataTransformer_1.StaticResourceMetadataTransformer(registry_1.mockRegistry);
            const expectedInfos = [
                {
                    source: fs.createReadStream(xml),
                    relativeDestination: path_1.join(type.directoryName, path_1.basename(xml)),
                },
            ];
            const extraInfo = [
                {
                    source: null,
                    relativeDestination: path_1.join(type.directoryName, 'a', 'b', 'c.css'),
                },
            ];
            const result = transformer.toSourceFormat(component);
            chai_1.expect(result.component).to.deep.equal(component);
            chai_1.expect(result.writeInfos).to.deep.equal(expectedInfos);
            chai_1.expect(result.getExtraInfos).to.be.a('function');
            chai_1.expect(yield result.getExtraInfos()).to.deep.equal(extraInfo);
        }));
        it('should work well for null contentType', () => {
            const component = mixedContentSingleFileConstants_1.MC_SINGLE_FILE_COMPONENT;
            const { type, content, xml } = component;
            env.stub(component, 'parseXml').returns({
                StaticResource: {
                    contentType: undefined,
                },
            });
            const transformer = new staticResourceMetadataTransformer_1.StaticResourceMetadataTransformer(registry_1.mockRegistry);
            const expectedInfos = [
                {
                    source: fs.createReadStream(content),
                    relativeDestination: path_1.join(type.directoryName, `${utils_1.baseName(content)}.bin`),
                },
                {
                    source: fs.createReadStream(xml),
                    relativeDestination: path_1.join(type.directoryName, path_1.basename(xml)),
                },
            ];
            chai_1.expect(transformer.toSourceFormat(component)).to.deep.equal({
                component,
                writeInfos: expectedInfos,
            });
        });
    });
});
//# sourceMappingURL=staticResourceMetadataTransformer.js.map