"use strict";
/*
 * Copyright (c) 2020, salesforce.com, inc.
 * All rights reserved.
 * Licensed under the BSD 3-Clause license.
 * For full license text, see LICENSE.txt file in the repo root or https://opensource.org/licenses/BSD-3-Clause
 */
Object.defineProperty(exports, "__esModule", { value: true });
const chai_1 = require("chai");
const metadata_registry_1 = require("../../src/metadata-registry");
const i18n_1 = require("../../src/i18n");
const registry_1 = require("../mock/registry");
const path_1 = require("path");
const errors_1 = require("../../src/errors");
const registryTestUtil_1 = require("./registryTestUtil");
const reginaConstants_1 = require("../mock/registry/reginaConstants");
const tarajiConstants_1 = require("../mock/registry/tarajiConstants");
const testUtil = new registryTestUtil_1.RegistryTestUtil();
describe('RegistryAccess', () => {
    const access = new metadata_registry_1.RegistryAccess(registry_1.mockRegistry);
    it('Should freeze the registry data parameter', () => {
        chai_1.expect(Object.isFrozen(access.registry)).to.be.true;
        chai_1.expect(Object.isFrozen(registry_1.mockRegistry)).to.be.false;
    });
    describe('getTypeFromName', () => {
        it('Should fetch type regardless of casing', () => {
            chai_1.expect(access.getTypeFromName('KeAnUReeVes')).to.deep.equal(registry_1.mockRegistry.types.keanureeves);
        });
        it('Should fetch type regardless of spaces', () => {
            chai_1.expect(access.getTypeFromName('kathy Bates')).to.deep.equal(registry_1.mockRegistry.types.kathybates);
        });
        it('Should throw an error if definition missing', () => {
            chai_1.assert.throws(() => access.getTypeFromName('TypeWithoutDef'), errors_1.TypeInferenceError, i18n_1.nls.localize('error_missing_type_definition', registry_1.mockRegistry.suffixes.missing));
        });
    });
    describe('getComponentsFromPath', () => {
        afterEach(() => testUtil.restore());
        describe('File Paths', () => {
            it('Should throw file not found error if given path does not exist', () => {
                const path = registry_1.keanu.KEANU_SOURCE_PATHS[0];
                chai_1.assert.throws(() => access.getComponentsFromPath(path), errors_1.TypeInferenceError, i18n_1.nls.localize('error_path_not_found', [path]));
            });
            it('Should determine type for metadata file with known suffix', () => {
                const path = registry_1.keanu.KEANU_XML_PATHS[0];
                const access = testUtil.createRegistryAccess([
                    {
                        dirPath: registry_1.keanu.KEANUS_DIR,
                        children: [registry_1.keanu.KEANU_SOURCE_NAMES[0], registry_1.keanu.KEANU_XML_NAMES[0]],
                    },
                ]);
                testUtil.stubAdapters([
                    {
                        type: registry_1.mockRegistry.types.keanureeves,
                        componentMappings: [
                            {
                                path,
                                component: registry_1.keanu.KEANU_COMPONENT,
                            },
                        ],
                    },
                ]);
                chai_1.expect(access.getComponentsFromPath(path)).to.deep.equal([registry_1.keanu.KEANU_COMPONENT]);
            });
            it('Should determine type for source file with known suffix', () => {
                const path = registry_1.keanu.KEANU_SOURCE_PATHS[0];
                const access = testUtil.createRegistryAccess([
                    {
                        dirPath: registry_1.keanu.KEANUS_DIR,
                        children: [registry_1.keanu.KEANU_SOURCE_NAMES[0], registry_1.keanu.KEANU_XML_NAMES[0]],
                    },
                ]);
                testUtil.stubAdapters([
                    {
                        type: registry_1.mockRegistry.types.keanureeves,
                        componentMappings: [{ path, component: registry_1.keanu.KEANU_COMPONENT }],
                    },
                ]);
                chai_1.expect(access.getComponentsFromPath(path)).to.deep.equal([registry_1.keanu.KEANU_COMPONENT]);
            });
            it('Should determine type for path of mixed content type', () => {
                const path = registry_1.taraji.TARAJI_SOURCE_PATHS[1];
                const access = testUtil.createRegistryAccess([
                    {
                        dirPath: path_1.dirname(path),
                        children: [path_1.basename(path)],
                    },
                ]);
                testUtil.stubAdapters([
                    {
                        type: registry_1.mockRegistry.types.tarajihenson,
                        componentMappings: [{ path, component: registry_1.taraji.TARAJI_COMPONENT }],
                    },
                ]);
                chai_1.expect(access.getComponentsFromPath(path)).to.deep.equal([registry_1.taraji.TARAJI_COMPONENT]);
            });
            it('Should determine type for path content files', () => {
                const path = registry_1.keanu.KEANU_SOURCE_PATHS[0];
                const access = testUtil.createRegistryAccess([
                    {
                        dirPath: path_1.dirname(path),
                        children: registry_1.keanu.KEANU_SOURCE_NAMES,
                    },
                ]);
                testUtil.stubAdapters([
                    {
                        type: registry_1.mockRegistry.types.keanureeves,
                        componentMappings: [{ path, component: registry_1.keanu.KEANU_CONTENT_COMPONENT }],
                        allowContent: false,
                    },
                ]);
                chai_1.expect(access.getComponentsFromPath(path)).to.deep.equal([registry_1.keanu.KEANU_CONTENT_COMPONENT]);
            });
            it('Should determine type for inFolder path content files', () => {
                const path = registry_1.sean.SEAN_FOLDER;
                const access = testUtil.createRegistryAccess([
                    {
                        dirPath: path,
                        children: registry_1.sean.SEAN_NAMES,
                    },
                ]);
                const componentMappings = registry_1.sean.SEAN_PATHS.map((p, i) => ({
                    path: p,
                    component: registry_1.sean.SEAN_COMPONENTS[i],
                }));
                testUtil.stubAdapters([
                    {
                        type: registry_1.mockRegistry.types.seanconnerys,
                        componentMappings,
                        allowContent: false,
                    },
                ]);
                chai_1.expect(access.getComponentsFromPath(path)).to.deep.equal(registry_1.sean.SEAN_COMPONENTS);
            });
            it('Should determine type for folder files', () => {
                const path = registry_1.gene.GENE_DIR;
                const access = testUtil.createRegistryAccess([
                    {
                        dirPath: path,
                        children: [registry_1.gene.GENE_FOLDER_XML_NAME],
                    },
                ]);
                testUtil.stubAdapters([
                    {
                        type: registry_1.mockRegistry.types.genewilder,
                        componentMappings: [
                            { path: registry_1.gene.GENE_FOLDER_XML_PATH, component: registry_1.gene.GENE_FOLDER_COMPONENT },
                        ],
                        allowContent: false,
                    },
                ]);
                chai_1.expect(access.getComponentsFromPath(path)).to.deep.equal([registry_1.gene.GENE_FOLDER_COMPONENT]);
            });
            it('Should not mistake folder component of a mixed content type as that type', () => {
                // this test has coveage on non-mixedContent types as well by nature of the execution path
                const path = registry_1.tina.TINA_FOLDER_XML;
                const access = testUtil.createRegistryAccess([
                    {
                        dirPath: registry_1.tina.TINA_DIR,
                        children: [path_1.basename(path)],
                    },
                ]);
                testUtil.stubAdapters([
                    {
                        type: registry_1.mockRegistry.types.tinafeyfolder,
                        componentMappings: [{ path, component: registry_1.tina.TINA_FOLDER_COMPONENT }],
                    },
                ]);
                chai_1.expect(access.getComponentsFromPath(path)).to.deep.equal([registry_1.tina.TINA_FOLDER_COMPONENT]);
            });
            it('Should throw type id error if one could not be determined', () => {
                const missing = path_1.join('path', 'to', 'whatever', 'a.b-meta.xml');
                const access = testUtil.createRegistryAccess([
                    {
                        dirPath: path_1.dirname(missing),
                        children: [path_1.basename(missing)],
                    },
                ]);
                chai_1.assert.throws(() => access.getComponentsFromPath(missing), errors_1.TypeInferenceError, i18n_1.nls.localize('error_could_not_infer_type', [missing]));
            });
            it('Should not return a component if path to metadata xml is forceignored', () => {
                const path = registry_1.keanu.KEANU_XML_PATHS[0];
                const access = testUtil.createRegistryAccess([
                    {
                        dirPath: path_1.dirname(path),
                        children: [path_1.basename(path)],
                    },
                ]);
                testUtil.stubForceIgnore({ seed: path, deny: [path] });
                testUtil.stubAdapters([
                    {
                        type: registry_1.mockRegistry.types.keanureeves,
                        // should not be returned
                        componentMappings: [{ path, component: registry_1.keanu.KEANU_COMPONENT }],
                    },
                ]);
                chai_1.expect(access.getComponentsFromPath(path).length).to.equal(0);
            });
            it('Should not return a component if path to content metadata xml is forceignored', () => {
                const path = registry_1.keanu.KEANU_XML_PATHS[0];
                const access = testUtil.createRegistryAccess([
                    {
                        dirPath: path_1.dirname(path),
                        children: [path_1.basename(path)],
                    },
                ]);
                testUtil.stubForceIgnore({ seed: path, deny: [path] });
                testUtil.stubAdapters([
                    {
                        type: registry_1.mockRegistry.types.keanureeves,
                        // should not be returned
                        componentMappings: [{ path, component: registry_1.keanu.KEANU_COMPONENT }],
                    },
                ]);
                chai_1.expect(access.getComponentsFromPath(path).length).to.equal(0);
            });
            it('Should not return a component if path to folder metadata xml is forceignored', () => {
                const path = registry_1.gene.GENE_FOLDER_XML_PATH;
                const access = testUtil.createRegistryAccess([
                    {
                        dirPath: path_1.dirname(path),
                        children: [path_1.basename(path)],
                    },
                ]);
                testUtil.stubForceIgnore({ seed: path, deny: [path] });
                testUtil.stubAdapters([
                    {
                        type: registry_1.mockRegistry.types.genewilder,
                        // should not be returned
                        componentMappings: [
                            { path: registry_1.gene.GENE_FOLDER_XML_PATH, component: registry_1.gene.GENE_FOLDER_COMPONENT },
                        ],
                    },
                ]);
                chai_1.expect(access.getComponentsFromPath(path).length).to.equal(0);
            });
        });
        describe('Directory Paths', () => {
            it('Should return all components in a directory', () => {
                const access = testUtil.createRegistryAccess([
                    {
                        dirPath: registry_1.kathy.KATHY_FOLDER,
                        children: registry_1.kathy.KATHY_XML_NAMES,
                    },
                ]);
                const componentMappings = registry_1.kathy.KATHY_XML_PATHS.map((p, i) => ({
                    path: p,
                    component: registry_1.kathy.KATHY_COMPONENTS[i],
                }));
                testUtil.stubAdapters([
                    {
                        type: registry_1.mockRegistry.types.kathybates,
                        componentMappings,
                    },
                ]);
                chai_1.expect(access.getComponentsFromPath(registry_1.kathy.KATHY_FOLDER)).to.deep.equal(registry_1.kathy.KATHY_COMPONENTS);
            });
            it('Should walk all file and directory children', () => {
                const { KEANUS_DIR } = registry_1.keanu;
                const stuffDir = path_1.join(KEANUS_DIR, 'hasStuff');
                const noStuffDir = path_1.join(KEANUS_DIR, 'noStuff');
                const kathyXml = path_1.join(KEANUS_DIR, registry_1.kathy.KATHY_XML_NAMES[0]);
                const keanuXml = registry_1.keanu.KEANU_XML_PATHS[0];
                const keanuSrc = registry_1.keanu.KEANU_SOURCE_PATHS[0];
                const keanuXml2 = path_1.join(stuffDir, registry_1.keanu.KEANU_XML_NAMES[1]);
                const keanuSrc2 = path_1.join(stuffDir, registry_1.keanu.KEANU_SOURCE_NAMES[1]);
                const tree = new metadata_registry_1.VirtualTreeContainer([
                    {
                        dirPath: KEANUS_DIR,
                        children: [
                            path_1.basename(keanuXml),
                            path_1.basename(keanuSrc),
                            registry_1.kathy.KATHY_XML_NAMES[0],
                            'hasStuff',
                            'noStuff',
                        ],
                    },
                    {
                        dirPath: noStuffDir,
                        children: [],
                    },
                    {
                        dirPath: stuffDir,
                        children: [path_1.basename(keanuSrc2), path_1.basename(keanuXml2)],
                    },
                ]);
                const keanuComponent2 = new metadata_registry_1.SourceComponent({
                    name: 'b',
                    type: registry_1.mockRegistry.types.keanureeves,
                    xml: keanuXml2,
                    content: keanuSrc2,
                }, tree);
                const kathyComponent2 = new metadata_registry_1.SourceComponent({
                    name: 'a',
                    type: registry_1.mockRegistry.types.kathybates,
                    xml: kathyXml,
                }, tree);
                const access = new metadata_registry_1.RegistryAccess(registry_1.mockRegistry, tree);
                testUtil.stubAdapters([
                    {
                        type: registry_1.mockRegistry.types.kathybates,
                        componentMappings: [
                            {
                                path: path_1.join(KEANUS_DIR, registry_1.kathy.KATHY_XML_NAMES[0]),
                                component: kathyComponent2,
                            },
                        ],
                    },
                    {
                        type: registry_1.mockRegistry.types.keanureeves,
                        componentMappings: [
                            {
                                path: keanuXml,
                                component: registry_1.keanu.KEANU_COMPONENT,
                            },
                            {
                                path: keanuXml2,
                                component: keanuComponent2,
                            },
                        ],
                    },
                ]);
                chai_1.expect(access.getComponentsFromPath(KEANUS_DIR)).to.deep.equal([
                    registry_1.keanu.KEANU_COMPONENT,
                    kathyComponent2,
                    keanuComponent2,
                ]);
            });
            it('Should handle the folder of a mixed content folder type', () => {
                const access = testUtil.createRegistryAccess([
                    {
                        dirPath: registry_1.tina.TINA_FOLDER,
                        children: registry_1.tina.TINA_XML_NAMES.concat(registry_1.tina.TINA_SOURCE_NAMES),
                    },
                ]);
                testUtil.stubAdapters([
                    {
                        type: registry_1.mockRegistry.types.tinafey,
                        componentMappings: [
                            {
                                path: registry_1.tina.TINA_XML_PATHS[0],
                                component: registry_1.tina.TINA_COMPONENTS[0],
                            },
                            {
                                path: registry_1.tina.TINA_XML_PATHS[1],
                                component: registry_1.tina.TINA_COMPONENTS[1],
                            },
                        ],
                    },
                ]);
                chai_1.expect(access.getComponentsFromPath(registry_1.tina.TINA_FOLDER)).to.deep.equal([
                    registry_1.tina.TINA_COMPONENTS[0],
                    registry_1.tina.TINA_COMPONENTS[1],
                ]);
            });
            it('Should return a component for a directory that is content or a child of content', () => {
                const { TARAJI_CONTENT_PATH } = registry_1.taraji;
                const access = testUtil.createRegistryAccess([
                    {
                        dirPath: TARAJI_CONTENT_PATH,
                        children: [],
                    },
                    {
                        dirPath: registry_1.taraji.TARAJI_DIR,
                        children: [registry_1.taraji.TARAJI_XML_NAMES[0], path_1.basename(TARAJI_CONTENT_PATH)],
                    },
                ]);
                testUtil.stubAdapters([
                    {
                        type: registry_1.mockRegistry.types.tarajihenson,
                        componentMappings: [
                            {
                                path: TARAJI_CONTENT_PATH,
                                component: registry_1.taraji.TARAJI_COMPONENT,
                            },
                        ],
                    },
                ]);
                chai_1.expect(access.getComponentsFromPath(TARAJI_CONTENT_PATH)).to.deep.equal([
                    registry_1.taraji.TARAJI_COMPONENT,
                ]);
            });
            it('Should not add duplicates of a component when the content has multiple -meta.xmls', () => {
                const { SIMON_COMPONENT, SIMON_BUNDLE_PATH } = registry_1.simon;
                const access = testUtil.createRegistryAccess([
                    {
                        dirPath: registry_1.simon.SIMON_DIR,
                        children: [path_1.basename(SIMON_BUNDLE_PATH)],
                    },
                    {
                        dirPath: SIMON_BUNDLE_PATH,
                        children: registry_1.simon.SIMON_SOURCE_PATHS.concat(registry_1.simon.SIMON_XML_PATH).map((p) => path_1.basename(p)),
                    },
                ]);
                testUtil.stubAdapters([
                    {
                        type: registry_1.mockRegistry.types.simonpegg,
                        componentMappings: [
                            { path: registry_1.simon.SIMON_BUNDLE_PATH, component: SIMON_COMPONENT },
                            { path: registry_1.simon.SIMON_XML_PATH, component: SIMON_COMPONENT },
                            {
                                path: registry_1.simon.SIMON_SUBTYPE_PATH,
                                component: SIMON_COMPONENT,
                            },
                        ],
                    },
                ]);
                chai_1.expect(access.getComponentsFromPath(registry_1.simon.SIMON_DIR)).to.deep.equal([SIMON_COMPONENT]);
            });
            it('Should not add duplicate component if directory content and xml are at the same level', () => {
                const access = testUtil.createRegistryAccess(tarajiConstants_1.TARAJI_VIRTUAL_FS);
                const component = metadata_registry_1.SourceComponent.createVirtualComponent(tarajiConstants_1.TARAJI_COMPONENT, tarajiConstants_1.TARAJI_VIRTUAL_FS);
                testUtil.stubAdapters([
                    {
                        type: registry_1.mockRegistry.types.tarajihenson,
                        componentMappings: [
                            { path: tarajiConstants_1.TARAJI_CONTENT_PATH, component },
                            { path: tarajiConstants_1.TARAJI_XML_PATHS[0], component },
                        ],
                    },
                ]);
                chai_1.expect(access.getComponentsFromPath(tarajiConstants_1.TARAJI_DIR)).to.deep.equal([component]);
            });
            it('Should stop resolution if parent component is resolved', () => {
                const access = testUtil.createRegistryAccess(reginaConstants_1.REGINA_VIRTUAL_FS);
                testUtil.stubAdapters([
                    {
                        type: registry_1.mockRegistry.types.reginaking,
                        componentMappings: [
                            { path: reginaConstants_1.REGINA_XML_PATH, component: reginaConstants_1.REGINA_COMPONENT },
                            { path: reginaConstants_1.REGINA_CHILD_XML_PATH_1, component: reginaConstants_1.REGINA_CHILD_COMPONENT_1 },
                        ],
                    },
                ]);
                chai_1.expect(access.getComponentsFromPath(reginaConstants_1.REGINA_PATH)).to.deep.equal([reginaConstants_1.REGINA_COMPONENT]);
            });
            /**
             * Because files of a mixed content type could have any suffix, they might collide
             * with a type that uses the "suffix index" in the registry and be assigned the incorrect type.
             *
             * Pretend that this bundle's root xml suffix is the same as KeanuReeves - still should be
             * identified as SimonPegg type
             */
            it('Should handle suffix collision for mixed content types', () => {
                const tree = new metadata_registry_1.VirtualTreeContainer([
                    {
                        dirPath: registry_1.simon.SIMON_DIR,
                        children: [path_1.basename(registry_1.simon.SIMON_BUNDLE_PATH)],
                    },
                    {
                        dirPath: registry_1.simon.SIMON_BUNDLE_PATH,
                        children: [registry_1.keanu.KEANU_XML_NAMES[0], path_1.basename(registry_1.simon.SIMON_SOURCE_PATHS[0])],
                    },
                ]);
                const access = new metadata_registry_1.RegistryAccess(registry_1.mockRegistry, tree);
                chai_1.expect(access.getComponentsFromPath(registry_1.simon.SIMON_DIR)).to.deep.equal([
                    new metadata_registry_1.SourceComponent({
                        name: 'a',
                        type: registry_1.mockRegistry.types.simonpegg,
                        xml: path_1.join(registry_1.simon.SIMON_BUNDLE_PATH, registry_1.keanu.KEANU_XML_NAMES[0]),
                        content: registry_1.simon.SIMON_BUNDLE_PATH,
                    }, tree),
                ]);
            });
            it('Should not return components if the directory is forceignored', () => {
                const dirPath = registry_1.kathy.KATHY_FOLDER;
                testUtil.stubForceIgnore({ seed: dirPath, deny: [dirPath] });
                const access = testUtil.createRegistryAccess([
                    {
                        dirPath,
                        children: [registry_1.kathy.KATHY_XML_NAMES[0], registry_1.kathy.KATHY_XML_NAMES[1]],
                    },
                ]);
                testUtil.stubAdapters([
                    {
                        type: registry_1.mockRegistry.types.kathybates,
                        componentMappings: [
                            {
                                path: registry_1.kathy.KATHY_XML_PATHS[0],
                                component: registry_1.kathy.KATHY_COMPONENTS[0],
                            },
                            {
                                path: registry_1.kathy.KATHY_XML_PATHS[1],
                                component: registry_1.kathy.KATHY_COMPONENTS[1],
                            },
                        ],
                    },
                ]);
                chai_1.expect(access.getComponentsFromPath(dirPath).length).to.equal(0);
            });
        });
    });
});
//# sourceMappingURL=registryAccess.test.js.map