"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
/*
 * Copyright (c) 2020, salesforce.com, inc.
 * All rights reserved.
 * Licensed under the BSD 3-Clause license.
 * For full license text, see LICENSE.txt file in the repo root or https://opensource.org/licenses/BSD-3-Clause
 */
const metadata_registry_1 = require("../../src/metadata-registry");
const registryTestUtil_1 = require("./registryTestUtil");
const registry_1 = require("../mock/registry");
const chai_1 = require("chai");
const reginaConstants_1 = require("../mock/registry/reginaConstants");
const keanuConstants_1 = require("../mock/registry/keanuConstants");
const sinon_1 = require("sinon");
const fs = require("fs");
const errors_1 = require("../../src/errors");
const i18n_1 = require("../../src/i18n");
const env = sinon_1.createSandbox();
describe('SourceComponent', () => {
    it('should return correct fullName for components without a parent', () => {
        chai_1.expect(reginaConstants_1.REGINA_COMPONENT.fullName).to.equal(reginaConstants_1.REGINA_COMPONENT.name);
    });
    describe('parseXml', () => {
        afterEach(() => env.restore());
        it('should parse the components xml file to json', () => {
            const component = keanuConstants_1.KEANU_COMPONENT;
            env.stub(fs, 'readFileSync').returns('<KeanuReeves><test>something</test></KeanuReeves>');
            chai_1.expect(component.parseXml()).to.deep.equal({
                KeanuReeves: {
                    test: 'something',
                },
            });
        });
        it('should throw an error if the component does not have an xml when parsing', () => {
            const component = new metadata_registry_1.SourceComponent({
                name: 'a',
                type: registry_1.mockRegistry.types.keanureeves,
            });
            chai_1.assert.throws(() => component.parseXml(), errors_1.LibraryError, i18n_1.nls.localize('error_parsing_xml', component.name));
        });
    });
    describe('walkContent', () => {
        it('should return empty array if no content is set', () => {
            const component = new metadata_registry_1.SourceComponent({
                name: 'a',
                type: registry_1.mockRegistry.types.kathybates,
                xml: registry_1.kathy.KATHY_XML_PATHS[0],
            });
            chai_1.expect(component.walkContent()).to.be.empty;
        });
        it('should return content if content is a file', () => {
            const component = metadata_registry_1.SourceComponent.createVirtualComponent(registry_1.keanu.KEANU_COMPONENT, [
                {
                    dirPath: registry_1.keanu.KEANUS_DIR,
                    children: [registry_1.keanu.KEANU_SOURCE_NAMES[0]],
                },
            ]);
            chai_1.expect(component.walkContent()).to.deep.equal([registry_1.keanu.KEANU_SOURCE_PATHS[0]]);
        });
        it('should collect all files if content is directory', () => {
            const component = metadata_registry_1.SourceComponent.createVirtualComponent({
                name: 'a',
                type: registry_1.mockRegistry.types.tarajihenson,
                xml: registry_1.taraji.TARAJI_XML_PATHS[0],
                content: registry_1.taraji.TARAJI_CONTENT_PATH,
            }, registry_1.taraji.TARAJI_VIRTUAL_FS);
            chai_1.expect(component.walkContent()).to.deep.equal(registry_1.taraji.TARAJI_SOURCE_PATHS);
        });
        it('Should not include source paths that are forceignored', () => {
            const testUtil = new registryTestUtil_1.RegistryTestUtil();
            const path = registry_1.taraji.TARAJI_SOURCE_PATHS[0];
            const forceIgnore = testUtil.stubForceIgnore({
                seed: path,
                accept: [registry_1.taraji.TARAJI_SOURCE_PATHS[1]],
                deny: [registry_1.taraji.TARAJI_SOURCE_PATHS[0], registry_1.taraji.TARAJI_SOURCE_PATHS[2]],
            });
            const component = metadata_registry_1.SourceComponent.createVirtualComponent(registry_1.taraji.TARAJI_COMPONENT, registry_1.taraji.TARAJI_VIRTUAL_FS, forceIgnore);
            chai_1.expect(component.walkContent()).to.deep.equal([registry_1.taraji.TARAJI_SOURCE_PATHS[1]]);
            testUtil.restore();
        });
    });
    describe('Child Components', () => {
        const type = registry_1.mockRegistry.types.reginaking;
        const expectedChild = metadata_registry_1.SourceComponent.createVirtualComponent({
            name: 'z',
            type: type.children.types.y,
            xml: registry_1.regina.REGINA_CHILD_XML_PATH_1,
            parent: registry_1.regina.REGINA_COMPONENT,
        }, registry_1.regina.REGINA_VIRTUAL_FS);
        const expectedChild2 = metadata_registry_1.SourceComponent.createVirtualComponent({
            name: 'w',
            type: type.children.types.x,
            xml: registry_1.regina.REGINA_CHILD_XML_PATH_2,
            parent: registry_1.regina.REGINA_COMPONENT,
        }, registry_1.regina.REGINA_VIRTUAL_FS);
        it('should return child components for a component', () => {
            chai_1.expect(registry_1.regina.REGINA_COMPONENT.getChildren()).to.deep.equal([expectedChild, expectedChild2]);
        });
        it('should not include children that are forceignored', () => {
            const testUtil = new registryTestUtil_1.RegistryTestUtil();
            const forceIgnore = testUtil.stubForceIgnore({
                seed: registry_1.regina.REGINA_XML_PATH,
                deny: [registry_1.regina.REGINA_CHILD_XML_PATH_2],
            });
            const component = metadata_registry_1.SourceComponent.createVirtualComponent(registry_1.regina.REGINA_COMPONENT, registry_1.regina.REGINA_VIRTUAL_FS, forceIgnore);
            chai_1.expect(component.getChildren()).to.deep.equal([expectedChild]);
            testUtil.restore();
        });
        it('should return correct fullName for components with a parent', () => {
            chai_1.expect(expectedChild.fullName).to.equal(`${registry_1.regina.REGINA_COMPONENT.name}.${expectedChild.name}`);
        });
        it('should return empty array if there is no metadata xml', () => {
            const noXml = metadata_registry_1.SourceComponent.createVirtualComponent({
                name: 'noXml',
                type: registry_1.mockRegistry.types.tinafey,
            }, []);
            chai_1.expect(noXml.getChildren()).to.be.empty;
        });
    });
});
//# sourceMappingURL=sourceComponent.test.js.map