"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
/*
 * Copyright (c) 2020, salesforce.com, inc.
 * All rights reserved.
 * Licensed under the BSD 3-Clause license.
 * For full license text, see LICENSE.txt file in the repo root or https://opensource.org/licenses/BSD-3-Clause
 */
const treeContainers_1 = require("../../src/metadata-registry/treeContainers");
const chai_1 = require("chai");
const sinon_1 = require("sinon");
const fs = require("fs");
const path_1 = require("path");
const errors_1 = require("../../src/errors");
const i18n_1 = require("../../src/i18n");
describe('Tree Containers', () => {
    const readDirResults = ['a.q', 'a.x-meta.xml', 'b', 'b.x-meta.xml', 'c.z', 'c.x-meta.xml'];
    describe('BaseTreeContainer', () => {
        class TestTreeContainer extends treeContainers_1.BaseTreeContainer {
            readDirectory() {
                return readDirResults;
            }
            exists() {
                return false;
            }
            isDirectory() {
                return false;
            }
            readFile() {
                return Promise.resolve(Buffer.from(''));
            }
        }
        const tree = new TestTreeContainer();
        it('should find first matching metadata file', () => {
            chai_1.expect(tree.find('metadata', 'b', '')).to.equal('b.x-meta.xml');
        });
        it('should find first matching content file', () => {
            chai_1.expect(tree.find('content', 'c', '')).to.equal('c.z');
        });
    });
    describe('NodeFSTreeContainer', () => {
        const env = sinon_1.createSandbox();
        const tree = new treeContainers_1.NodeFSTreeContainer();
        const path = path_1.join('path', 'to', 'test');
        afterEach(() => env.restore());
        it('should use expected Node API for isDirectory', () => {
            const statStub = env.stub(fs, 'lstatSync');
            // @ts-ignore lstat returns more than isDirectory function
            statStub.withArgs(path).returns({ isDirectory: () => true });
            chai_1.expect(tree.isDirectory(path)).to.be.true;
            chai_1.expect(statStub.calledOnce).to.be.true;
        });
        it('should use expected Node API for exists', () => {
            const existsStub = env.stub(fs, 'existsSync');
            existsStub.withArgs(path).returns(true);
            chai_1.expect(tree.exists(path)).to.be.true;
            chai_1.expect(existsStub.calledOnce).to.be.true;
        });
        it('should use expected Node API for readDirectory', () => {
            const readdirStub = env.stub(fs, 'readdirSync');
            // @ts-ignore wants Dirents but string[] works as well
            readdirStub.withArgs(path).returns(readDirResults);
            chai_1.expect(tree.readDirectory(path)).to.deep.equal(readDirResults);
            chai_1.expect(readdirStub.calledOnce).to.be.true;
        });
        it('should use expected Node API for readFile', () => __awaiter(void 0, void 0, void 0, function* () {
            const readFileStub = env.stub(fs.promises, 'readFile');
            // @ts-ignore wants Dirents but string[] works as well
            readFileStub.withArgs(path).resolves(Buffer.from('test'));
            const data = yield tree.readFile(path);
            chai_1.expect(data.toString()).to.deep.equal('test');
            chai_1.expect(readFileStub.calledOnce).to.be.true;
        }));
    });
    describe('VirtualTreeContainer', () => {
        const virtualFS = [
            {
                dirPath: '.',
                children: ['test.txt', 'test2.txt', 'files', 'logs'],
            },
            {
                dirPath: path_1.join('.', 'files'),
                children: ['test3.txt'],
            },
            {
                dirPath: path_1.join('.', 'logs'),
                children: [{ name: 'run.log', data: Buffer.from('successful') }],
            },
        ];
        const tree = new treeContainers_1.VirtualTreeContainer(virtualFS);
        describe('isDirectory', () => {
            it('should return false for isDirectory', () => {
                const path = path_1.join('.', 'test.txt');
                chai_1.expect(tree.isDirectory(path)).to.be.false;
            });
            it('should return true for isDirectory', () => {
                const path = path_1.join('.', 'files');
                chai_1.expect(tree.isDirectory(path)).to.be.true;
            });
            it('should throw an error if path does not exist', () => {
                const path = 'dne';
                chai_1.assert.throws(() => tree.isDirectory(path), errors_1.LibraryError, i18n_1.nls.localize('error_path_not_found', path));
            });
        });
        describe('exists', () => {
            it('should return true for file that exists', () => {
                const path = path_1.join('.', 'test.txt');
                chai_1.expect(tree.exists(path)).to.be.true;
            });
            it('should return true for directory that exists', () => {
                const path = path_1.join('.', 'files');
                chai_1.expect(tree.exists(path)).to.be.true;
            });
            it('should return false for file that does not exist', () => {
                const path = path_1.join('.', 'files', 'text4.txt');
                chai_1.expect(tree.exists(path)).to.be.false;
            });
            it('should return false for directory that does not exist', () => {
                const path = path_1.join('.', 'otherfiles');
                chai_1.expect(tree.exists(path)).to.be.false;
            });
        });
        it('should return directory entries for readDirectory', () => {
            chai_1.expect(tree.readDirectory('.')).to.deep.equal(virtualFS[0].children);
        });
        describe('readFile', () => {
            it('should return file data for given path', () => __awaiter(void 0, void 0, void 0, function* () {
                const data = yield tree.readFile(path_1.join('.', 'logs', 'run.log'));
                chai_1.expect(data.toString()).to.equal('successful');
            }));
            it('should return empty string buffer if file initialize without data', () => __awaiter(void 0, void 0, void 0, function* () {
                const data = yield tree.readFile(path_1.join('.', 'test.txt'));
                chai_1.expect(data.toString()).to.equal('');
            }));
            it('should throw error if path does not exist', () => __awaiter(void 0, void 0, void 0, function* () {
                const path = 'dne';
                try {
                    yield tree.readFile(path);
                    chai_1.assert.fail('should have thrown an error');
                }
                catch (e) {
                    chai_1.expect(e.message).to.deep.equal(i18n_1.nls.localize('error_path_not_found', path));
                }
            }));
        });
    });
});
//# sourceMappingURL=treeContainers.test.js.map